/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.description;

import java.util.Iterator;
import java.util.ArrayList;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * Describes an Array of source elements (i.e. elements which provide a value).
 */
public class ArraySource extends InlinedElement {
	private ArrayList arrayElements;
	private String typeName;
	
	/**
	 * The public constructor of an array source.
	 * @param aTypeName : a type is defined here through its name (via a simple String and not via a Type).
	 * No semantical check is performed making it possible to use arbitrary types (java types, for example).
	 * A simple syntactical check is done however : name must be of the form : IDENT([])*
	 * examples of correct type names are : int[ ], toto[ ][ ].
	 * @param aTemplate : the template containing the array definition.
	 * @throws KilimException : generated in case of bad syntactical form.
	 */
	public ArraySource(String aTypeName, TemplateDescription aTemplate) throws KilimException {
		super(true, false, aTemplate);
		checkTypeName(aTypeName);
		typeName = aTypeName;
	}
	
	/**
	 * performs a simple syntactical check of the type name:
	 * The name must be of the form : IDENT([])*. examples of correct type names are : int[], toto[][].
	 */
	private void checkTypeName(String aTypeName) throws KilimException {
		String name = aTypeName;
		
		while (name.endsWith("[]")) {
			name = name.substring(0, name.length() - 2);
		}
		if (name.indexOf("[") != -1 || name.indexOf("]") != -1) {
			throw new KilimException("syntax error in type declaration " + aTypeName + " of an array in template " + getContainingTemplate().getName());
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.ARRAY;
	}

	/**
	 * returns the current size of the array.
	 * @return int.
	 */
	public int getCurrentSize() {
		if (arrayElements == null) {
			return 0;
		}
		return arrayElements.size();
	}
	
	/**
	 * extends an array by adding a new element in the array.
	 * @param value : must be a basic template element
	 * @throws KilimException : generated when the argument is null or when the argument does not provide
	 * a value.
	 */
	public void addElement(BasicElement value) throws KilimException {
		if (value == null) {
			throw new KilimException("attempt to add a null value in an array in template " + getContainingTemplate().getName());
		}
		
		if (!value.providesValue()) {
			throw new KilimException("attempt to add a non provider to an array in template " + getContainingTemplate().getName());
		}
		if (arrayElements == null) {
			arrayElements = new ArrayList();
		}
		arrayElements.add(value);
	}
	
	/**
	 * returns the element stored at a given position.
	 * @param aIndex : index in the array
	 * @return BasicElement : the element at the indicated position.
	 * @throws KilimException : generated for illegal values of index (i.e. negative values, or values exceding the size of the array).
	 */
	public BasicElement getElement(int aIndex) throws KilimException {
		if (aIndex < 0) {
			throw new KilimException("attempt to use an illegal index : " + aIndex + " in getting Element of an array in template " + getContainingTemplate().getName());
		}
		return (BasicElement) arrayElements.get(aIndex);
	}
	
	/**
	 * returns the elements of the array as an iterator.
	 * @return Iterator
	 */
	public Iterator getElements() {
		if (arrayElements == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return arrayElements.listIterator();
	}
	
	/**
	 * @see org.objectweb.kilim.description.Array#getType()
	 */
	public String getTypeName() {
		return typeName; 
	}
}