/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public abstract class BasicNamedElementImpl extends TemplateElementImpl implements BasicNamedElement {	
	private String localName;
	private int status;
	private boolean isProvider;
	private boolean isTransformer;
	
	/**
	 * This constructor is used internally by the kilim infrastructure.
	 * @param aName : the name of the basic element.
	 * @param aStatus : an int representing the status : should be one of KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 * @param isP :is true when the element provides a value (i.e. act as a provider).
	 * @param isT : is true when the element performs an action (i.e. act as a transformer).
	 * @param aTemplate : the template containing the element.
	 * @throws KilimException : generatyed when the name reference is null, when the template reference is null,
	 * when the status value is illegal.
	 */
	protected BasicNamedElementImpl(String aName, int aStatus, boolean isP, boolean isT, TemplateDescription aTemplate) throws KilimException {
		if (aName == null) {
			if (aTemplate == null) {
				throw new KilimException("attempt to create a named element with a null name in a null template ");
			} else {
				throw new KilimException("attempt to create a named element with a null name in template " + aTemplate.getName());
			}
		}
		if (aTemplate == null) {
			throw new KilimException("attempt to create a named element " + aName + " in a null template");
		}
		checkStatus(aStatus);
		localName = aName;
		status = aStatus;
		isProvider = isP;
		isTransformer = isT;		
		setContainingTemplate(aTemplate);
	}

	/**
	 * This constructor is used internally by the kilim infrastructure. No check is performed.
	 * @param aName : the name of the basic element.
	 * @param aStatus : an int representing the status : should be one of KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 * @param isP :is true when the element provides a value (i.e. act as a provider).
	 * @param isT : is true when the element performs an action (i.e. act as a transformer).
	 */
	
	protected BasicNamedElementImpl(String aName, int aStatus, boolean isP, boolean isT) {
		localName = aName;
		status = aStatus;
		isProvider = isP;
		isTransformer = isT;
	}

	/**
	 * @see org.objectweb.kilim.description.TemplateElement#getLocalName()
	 */
	public String getLocalName() {
		return localName;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setLocalName(String)
	 */

	public void setLocalName(String aName) throws KilimException {
		if (aName == null) {
			if (getContainingTemplate() == null) {
				throw new KilimException("attempt to set a null name to element " + localName + " in a null template ");
			} else {
				throw new KilimException("attempt to set a null name to element " + localName + " in template " + getContainingTemplate().getName());			}
		}
		localName = aName;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#getStatus()
	 */
	public int getStatus() {
		return status;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setStatus(int)
	 */
	public void setStatus(int aStatus) throws KilimException {
		checkStatus(aStatus);
		status = aStatus;
	}
			
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#providesValue()
	 */
	public boolean providesValue() {
		return isProvider;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#performsAction()
	 */
	public boolean performsAction() {
		return isTransformer;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}

	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setContainingTemplate(TemplateDescription)
	 */
	public void setContainingTemplate(TemplateDescription aTemplate) throws KilimException {
		if (aTemplate == null) {
			throw new KilimException("attempt to set a null template to element " + localName + " in template " + getContainingTemplate().getName());
		}
		super.setContainingTemplate(aTemplate);
	}	
				
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return getLocalName() + "in template " + getContainingTemplate();
	}
	
	private void checkStatus(int aStatus) throws KilimException {
		if (aStatus < 1 || aStatus > 3) {
			throw new KilimException("illegal value " + aStatus + " for element " + localName);
		}
	}
}