/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import java.util.LinkedHashMap;
import java.util.Iterator;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * Describes a slot, which is just a set of unbound ports in a new naming context
 */
public class Plug extends TemplateElementImpl {
	private static String[][] msgTexts = {
		{ "attempt to create a plug with a null slot name", "" },																//0
		{ "attempt to plug a null instance name ", "" },																			//1
		{ "attempt to define a plug to a slot ", " in a null template" },														//2
		{ "attempt to add a mapping with a null external name in plug of slot ", "" },								//3
		{ "attempt to add an already added name mapping : ", " in slot " },												//4
		{ "attempt to remove a null nam mapping from a plug to slot ", "" },												//5
		{ "attempt to remove a name mapping from a plug to ", " without name mapping in template " }	//6
	};
	
	private String slotName;
	private String instanceName;
	private LinkedHashMap mappings;
	
	/**
	 * Method Plug.
	 * @param aSlotName : the name of the slot
	 * @param aInstanceName : the name of the component to be plugged.
	 * @param aTemplate : the template in which the plug is done.
	 * @throws KilimException : generates an exception if an argument is null.
	 */
	public Plug(String aSlotName, String aInstanceName, TemplateDescription aTemplate) throws KilimException {
		if (aSlotName == null) {
			throw new KilimException(msgTexts[0][0]);
		}
		if (aInstanceName == null) {
			throw new KilimException(msgTexts[1][0] + msgSuffix1());
		}
		if (aTemplate == null) {
			throw new KilimException(msgTexts[2][0] + aSlotName + msgTexts[2][1]);
		}		
			
		slotName = aSlotName;
		instanceName = aInstanceName;
		setContainingTemplate(aTemplate);
	}
		
	/**
	 * returns the slot name.
	 * @return String
	 */
	public String getSlotName() {
		return slotName;
	}
	
	/**
	 * returns the component name.
	 * @return String
	 */
	public String getInstanceName() {
		return instanceName;
	}
		
	/**
	 * A name mapping is a correspondance between the name of an interface defined in a slot and the corresponding port in a component.
	 * This correspondance has been introduced to relax constraints on the interface naming during plug operations. This method returns the
	 * name of component interface that will be associate to interface aName in the slot. 
	 * @param aName : the local name of the interface declared in the slot.
	 * @return String the local name of the corresponding interface in the component.
	 */
	public String getNameMapping(String aName) {
		if (mappings == null) {
			return aName;
		}
		
		Object name1 = mappings.get(aName);
		if (name1 == null) {
			return aName;
		} else {
			return (String) name1;
		}
	}
	
	/**
	 * A name mapping is a correspondance between the name of an interface defined in a slot and the corresponding port in a component.
	 * This correspondance has been introduced to relax constraints on the interface naming during plug operations. This method returns as 
	 * an iterator the name mappings between slot ports and the plugged component ports.
	 * @return Iterator
	 */
	public Iterator getNameMappings() {
		if (mappings == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return mappings.values().iterator();
	}

	/**
	 * A name mapping is a correspondance between the name of an interface defined in a slot and the corresponding port in a component.
	 * This correspondance has been introduced to relax constraints on the interface naming during plug operations. This method adds a new name
	 *  mapping.
	 * @param aExternal :name of the slot port
	 * @param aInternal : name of the component port.
	 * @throws KilimException : generated when aExternal or aInternal is null
	 */
	public void addNameMapping(String aExternal, String aInternal) throws KilimException {
		if (aExternal == null) {
			throw new KilimException(msgTexts[3][0] + msgSuffix1());
		}
		if (mappings == null) {
			mappings = new LinkedHashMap();
		}
		
		if (mappings.containsKey(aExternal)) {
			throw new KilimException(msgTexts[4][0] + aExternal + " to " + aInternal + msgTexts[4][1] + msgSuffix1());
		}
		mappings.put(aExternal, aInternal);
	}
	
	/**
	 * A name mapping is a correspondance between the name of an interface defined in a slot and the corresponding port in a component.
	 * This correspondance has been introduced to relax constraints on the interface naming during plug operations. This method removes a name 
	 * mapping.
	 * @param aExternal : name of the mapping to be removed (name mapping are identified by the slot name of a port).
	 * @throws KilimException : generated if aExternal is null.
	 */
	public void removeNameMapping(String aExternal) throws KilimException {
		if (aExternal == null) {
			throw new KilimException(msgTexts[5][0] + msgSuffix1());
		}
		
		if (mappings == null) {
			throw new KilimException(msgTexts[6][0] + slotName + msgTexts[6][1] + msgSuffix1());
		}
		mappings.remove(aExternal);
	}
	
	private String msgSuffix1() {
		return slotName + " in template " + getContainingTemplate();
	}
}