/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class Port extends BasicNamedElementImpl {

	/** a constant associated to the "this" port, a port bound to the Component object */ 
	public static final Port THIS_PORT = new Port();
		
	private Arity arity;
	private boolean isOffered;
	
	/**
	 * The public constructor for a port.
	 * @param aName : the local name of the port.
	 * @param aArity : the arity of the port.
	 * @param aStatus : the status of the port. It should be one of KILIM.PRIVATE, KILIM.PROTECTED, KILIM.PUBLIC.
	 * @param isP : true if the port is offered, false if not.
	 * @param aTemplate : the template in which the port is defined.
	 * @throws KilimException : generated if aName or aTemplate is null, or if aStatus has an illegal value, 
	 */
	public Port(String aName, Arity aArity, int aStatus, boolean isP, TemplateDescription aTemplate) throws KilimException {
		super(aName, aStatus, true, false, aTemplate);
		isOffered = isP;
		arity = aArity;
	}
	
	private Port() {
		super("THIS", KILIM.PUBLIC, true, false);
		isOffered = true;
		arity = Arity.REQUIRED;
	}
	
	/**
	 * returns whether the port is offered or not.
	 * @return boolean
	 */
	public boolean isOffered() {
		return isOffered;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.PORT;
	}
	
	/**
	 * @see org.objectweb.kilim.description.NamedElement#setLocalName(String)
	 */
	public void setLocalName(String aName) throws KilimException {
		String oName = getLocalName();
		super.setLocalName(aName);
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalPort(oName);
			template.addLocalPort(this);
		}
	}
		
	/**
	 * @see org.objectweb.kilim.description.NamedElement#setStatus(int)
	 */
	public void setStatus(int aStatus) throws KilimException {
		super.setStatus(aStatus);
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalPort(getLocalName());
			template.addLocalPort(this);
		}
	}	
	
	/**
	 *	@see org.objectweb.kilim.description.Port#isOptional()
	 */
	public boolean isOptional() {
		return arity == Arity.OPTIONAL;
	}
		
	/**
	 * @see org.objectweb.kilim.description.Port#isUnary()
	 */
	public boolean isUnary() {
		return arity != Arity.COLLECTION;
	}
	
	/**
	 * Returns the arity.
	 * @return Arity
	 */	
	public Arity getArity() {
		return arity;
	}
	
	/**
	 * Sets the arity.
	 * @param anArity The arity to set
	 * @throws KilimException :
	 */
	public void setArity(Arity anArity) throws KilimException {
		if (anArity == null) {
			throw new KilimException("attempt to set a null arity to port " + getLocalName() + " in template " + getContainingTemplate().getName());
		}
		arity = anArity;
	}
}