/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;
import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class Reference extends TemplateElementImpl  implements BasicElement {
	private String targetName;
	private BasicNamedElement boundElement;
	private boolean performsAction;
	private boolean providesValue;

	/**
	 * a public constructor for references..
	 * @param aTargetName : the name of the target
	 * @param aTemplate : the template in which the reference is defined
	 * @param isP : true if the reference target provides a value (and can thus be used as a provider).
	 * @param isT : true if the reference target executes an action (and can thus be used as a transformer).
	 * @throws KilimException : generated if aTargetName or aTemplate is null.
	 */
	public Reference(String aTargetName, TemplateDescription aTemplate, boolean isP, boolean isT) throws KilimException {
		if (aTemplate == null) {
			throw new KilimException("null template when constructing a reference");
		}
		if (aTargetName == null) {
			throw new KilimException("null Name when constructing a reference in template " + aTemplate.getName());
		}
		targetName = aTargetName;
		providesValue = isP;
		performsAction = isT;
		setContainingTemplate(aTemplate);
	}
	
	/**
	 * sets the name of a target for the reference. Setting a new name resets the boundElement to null.
	 * @param aName : a name for the target.
	 */
	public void setTargetName(String aName) {
		if (aName == targetName) {
			return;
		}
		targetName = aName;
		boundElement = null;
	}
	
	/**
	 * returns the name of the target.
	 * @return String
	 */
	public String getTargetName() {
		return targetName;
	}
		
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setContainingTemplate(TemplateDescription)
	 */
	public void setContainingTemplate(TemplateDescription aTemplate) throws KilimException {
		if (aTemplate == null) {
			throw new KilimException("attempt to set a null template to a reference to " + targetName + "in template " + getContainingTemplate());
		}
		super.setContainingTemplate(aTemplate);
	}
		
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		if (targetName == null) {
			return "[non initialized reference in template " + getContainingTemplate() + "]";
		}
		return "[" + targetName + "]";
	}
			
	/**
	 * @see org.objectweb.kilim.description.ExtendedSource#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#providesValue()
	 */
	public boolean providesValue() {
		return providesValue;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#performsAction()
	 */
	public boolean performsAction() {
		return performsAction;
	}
	
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.REFERENCE;
	}
}
