/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.description;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Set;
import java.util.Iterator;
import java.util.List;
import java.util.ArrayList;

import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.repository.ResourceLoader;

/**
 * @author horn
 * This is the class for Kilim templates.
 */

public class TemplateDescription {

	private static final Object LOCK = new Object();
	
	private static final int PROVIDER_REFERENCE = 1;
	private static final int TRANSFORMER_REFERENCE = 2;

	private static final int PROVIDER_KEY = 1;
	private static final int TRANSFORMER_KEY = 2;
	private static final int PROPERTY_KEY = 3;
	private static final int INSTANCE_KEY = 4;
	private static final int PORT_KEY = 5;
	private static final int TRIGGER_KEY = 6;
	private static final int BINDING_KEY = 7;
	private static final int SLOT_KEY = 8;
	private static final int PLUG_KEY = 9;
	private static final int ANY_PROVIDER_KEY = 10;
	
	private static final int LEGAL_OVERRIDE = 1;
	private static final int ILLEGAL_OVERRIDE = 2;
	private static final int NAME_CLASH = 3;
	private static final int NO_OVERRIDE = 4;	
	
	private String name;
	private TemplateDescription superTemplate;
	
	private LinkedHashMap transformers;
	private LinkedHashMap instances;
	private LinkedHashMap slots;
	private LinkedHashMap references;
	private LinkedHashMap allProviders;
	
	private Set subTemplates;	
	private List triggers;
	private List plugs;	
	private List bindings;
	
	private ResourceLoader resourceLoader;
	
	private class ReferenceData {
		int referenceKind;
		Object reference;
		
		ReferenceData(int aKind, Object aReference) {
			referenceKind = aKind;
			reference = aReference;
		}
	}
	
	private class ProviderData {
		int referenceKind;
		NamedElement provider;
		
		ProviderData(int aKind, NamedElement aProvider) {
			referenceKind = aKind;
			provider = aProvider;
		}
	}
		
	/**
	 * A public constructor to be used to build a template extending an existing template
	 * @param aName : the name of the template (which must be non null)
	 * @param aSuperTemplate : the reference of the super template (xhich can be null)
	 * @throws KilimException : thrown when name is null.
	 */
	public TemplateDescription(String aName, TemplateDescription aSuperTemplate) throws KilimException {
		if (aName == null) {
			if (superTemplate != null) {
				throw new KilimException("attempt to create a sub-template with a null name in template " + aSuperTemplate);
			} else {
				throw new KilimException("attempt to create a sub-template with a null name ");
			}
		}
		name = aName;
		
		superTemplate = aSuperTemplate;		
		if (superTemplate != null) {
			superTemplate.addSubTemplate(this);
		}
	}
	
	/**
	 * To be used to build a top level template (i.e. a template without super template).
	 * @param aName : the  name of the template (which must be non null)
	 * @throws KilimException : thrown when name is null
	 */
	public TemplateDescription(String aName) throws KilimException {
		this(aName, null);
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#getName()
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#setName(String)
	 */
	public void setName(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("illegal null name in setName of TemplateImpl");
		}
		name = aName;
	}
		
	/**
	 * @see org.objectweb.kilim.description.Template#getSuperTemplate()
	 */
	public TemplateDescription getSuperTemplate() {
		return superTemplate;
	}	

	/**
	 * @see org.objectweb.kilim.description.Template#setSuperTemplate(Template)
	 */
	public void setSuperTemplate(TemplateDescription aTemplate) throws KilimException {
		if (superTemplate != null) {
			superTemplate.removeSubTemplate(this);
		}
		superTemplate = aTemplate;
		if (superTemplate != null) {
			superTemplate.addSubTemplate(this);
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#addSubTemplate(Template)
	 */
	public void addSubTemplate(TemplateDescription aTemplate) throws KilimException {
		if (aTemplate == null) {
			throw new KilimException("null template in addSubTemplate");
		}
		if (subTemplates == null) {
			subTemplates = new HashSet();
		}
		subTemplates.add(aTemplate);
		
		Iterator iter = getPorts(false);
		while (iter.hasNext()) {
			NamedElement element = (NamedElement) iter.next();
			addSubTemplate1(element, PORT_KEY, aTemplate);
		}
		iter = getProperties(false);
		while (iter.hasNext()) {
			NamedElement element = (NamedElement) iter.next();
			addSubTemplate1(element, PROPERTY_KEY, aTemplate);
		}
		iter = getProviders(false);
		while (iter.hasNext()) {
			NamedElement element = (NamedElement) iter.next();
			addSubTemplate1(element, PROVIDER_KEY, aTemplate);
		}
		iter = getTransformers(false);
		while (iter.hasNext()) {
			NamedElement element = (NamedElement) iter.next();
			addSubTemplate1(element, TRANSFORMER_KEY, aTemplate);
		}
	}
	
	private void addSubTemplate1(NamedElement element, int tabKey, TemplateDescription aTemplate) throws KilimException {
		String eName = element.getLocalName();
		int status = element.getStatus();
		boolean canOverride = (status == KILIM.PUBLIC || status == KILIM.PROTECTED);
		int result = aTemplate.checkDownOverride(eName, tabKey, canOverride);
		if (result == NAME_CLASH) {
			throw new KilimException("setting template " + name + " as super template of " + aTemplate + " leads to a name clash for" + eName);
		}
		if (result == ILLEGAL_OVERRIDE) {
			throw new KilimException("setting template " + name + " as super template of " + aTemplate + " leads to an illegal override" + eName);				
		}
	}
		
	/**
	 * @see org.objectweb.kilim.description.Template#removeSubTemplate(Template)
	 */
	public void removeSubTemplate(TemplateDescription aTemplate) throws KilimException {
		if (aTemplate == null) {
			throw new KilimException("illegal null template in removeSubTemplate");
		}
		subTemplates.remove(aTemplate);			
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#getSubTemplates()
	 */
	public Iterator getSubTemplates() {
		if (subTemplates == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return subTemplates.iterator();
	}
	
	/**
	 * Method getTemplateDefHierarchy returns an iterator providing all the template elements overriden by a given element.
	 * The iterator contains the base element used for the look up.
	 * @param aElement : the base element used for the look up.
	 * @return Iterator
	 * @throws KilimException :
	 */
	public Iterator getTemplateDefHierarchy(TemplateElementImpl aElement) throws KilimException {
		String lName = null;
		if (aElement instanceof Port) {
			lName = ((Port) aElement).getLocalName();
			return getMapDefHierarchy(lName, PORT_KEY);
		}
		if (aElement instanceof Property) {
			lName = ((Property) aElement).getLocalName();
			return getMapDefHierarchy(lName, PROPERTY_KEY);
		}
		if (aElement instanceof Provider) {
			lName = ((Provider) aElement).getLocalName();
			return getMapDefHierarchy(lName, PROVIDER_KEY);
		}
		if (aElement instanceof Transformer) {
			lName = ((Transformer) aElement).getLocalName();
			return getMapDefHierarchy(lName, TRANSFORMER_KEY);
		}
		if (aElement instanceof Slot) {
			lName = ((Slot) aElement).getLocalName();
			return getMapDefHierarchy(lName, SLOT_KEY);
		}
		if (aElement instanceof Instance) {
			lName = ((Instance) aElement).getLocalName();
			return getMapDefHierarchy(lName, INSTANCE_KEY);
		}
		if (aElement instanceof Binding) {
			lName = ((Binding) aElement).getPortName();
			return getListDefHierarchy(lName, BINDING_KEY);
		}
		if (aElement instanceof Trigger) {
			lName = ((Trigger) aElement).getSourceName();
			return getListDefHierarchy(lName, TRIGGER_KEY);
		}
		if (aElement instanceof Plug) {
			lName = ((Plug) aElement).getSlotName();
			return getListDefHierarchy(lName, PLUG_KEY);
		}
		return KILIM.EMPTY_ITERATOR;
	}
	
	/**
	 * Method getAllMapContainingTemplates.
	 * @param aName :
	 * @param tabKey :
	 * @return Iterator
	 * @throws KilimException :
	 */
	private Iterator getMapDefHierarchy(String aName, int tabKey)  throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get the containing templates of an element through a null name");
		}
		
		TemplateDescription tmp0 = this;
		List list = new ArrayList();
		do {
			NamedElement  elem = tmp0.getLocalMapElement(aName, tabKey);
			if (elem != null) {
				list.add(tmp0);
			}
		} while ((tmp0 = tmp0.getSuperTemplate()) != null);
		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return list.iterator();
	}

		
	private Iterator getListDefHierarchy(String aName, int tabKey)  throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get the containing templates of an element through a null name");
		}
		
		TemplateDescription tmp0 = this;
		List list = new ArrayList();
		do {
			Iterator  iter = tmp0.getLocalListElements(aName, tabKey);
			if (iter != KILIM.EMPTY_ITERATOR) {
				list.add(tmp0);
			}
		} while ((tmp0 = tmp0.getSuperTemplate()) != null);
		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return list.iterator();
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#getPort(String, boolean)
	 */
	public BasicNamedElement getAnyProvider(String aName, boolean onlyLocal) throws KilimException {
		LinkedHashMap map = getMap(ANY_PROVIDER_KEY, false);
		BasicNamedElement provider = null;
		if (map != null) {
			ProviderData data = (ProviderData) map.get(aName);
			if (data != null) {
				provider = (BasicNamedElement) data.provider;
			}
		}
		if (provider != null || onlyLocal || superTemplate == null) {
			return provider;
		}
		return (BasicNamedElement) getSuperTemplate().getRMapElement(aName, ANY_PROVIDER_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getProviderMap(boolean)
	 */
	public Iterator getProviders(boolean onlyLocal) throws KilimException {
		List list = new ArrayList();
		Iterator iter = getAllInterfaces(onlyLocal);
		while (iter.hasNext()) {
			Object elem = iter.next();
			if (elem instanceof Provider) {
				list.add(elem);
			}
		}

		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return list.iterator();
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getProviderMap(boolean)
	 */	
	public Iterator getPorts(boolean onlyLocal) throws KilimException {
		List list = new ArrayList();
		Iterator iter = getAllInterfaces(onlyLocal);
		while (iter.hasNext()) {
			Object elem = iter.next();
			if (elem instanceof Port) {
				list.add(elem);
			}
		}

		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return list.iterator();
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getProviderMap(boolean)
	 */	
	public Iterator getProperties(boolean onlyLocal) throws KilimException {
		List list = new ArrayList();
		Iterator iter = getAllInterfaces(onlyLocal);
		while (iter.hasNext()) {
			Object elem = iter.next();
			if (elem instanceof Property) {
				list.add(elem);
			}
		}

		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return list.iterator();
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getPort(String, boolean)
	 */
	public Slot getSlot(String aName, boolean onlyLocal) throws KilimException {
		LinkedHashMap map = getMap(SLOT_KEY, false);
		Slot slot = null;
		if (map != null) {
			slot = (Slot) map.get(aName);
		}
			
		if (slot != null || onlyLocal || superTemplate == null) {
			return slot;
		}

		return (Slot) getSuperTemplate().getRMapElement(aName, SLOT_KEY);
	}
		
	/**
	 * @see org.objectweb.kilim.description.Template#getPortMap(boolean)
	 */
	public Iterator getSlots(boolean onlyLocal) throws KilimException {
		if (onlyLocal) {
			if (slots == null) {
				return KILIM.EMPTY_ITERATOR;
			}
			return slots.values().iterator();
		}
		
		LinkedHashMap map = new LinkedHashMap();
		addMapVisible(map, SLOT_KEY, false);
		return map.values().iterator();		
	}
					
	/**
	 * @see org.objectweb.kilim.description.Template#getTransformer(String, boolean)
	 */
	public BasicNamedElement getTransformer(String aName, boolean onlyLocal) throws KilimException  {
		LinkedHashMap map = getMap(TRANSFORMER_KEY, false);
		BasicNamedElement transformer = null;
		if (map != null) {
			transformer =  (BasicNamedElement) map.get(aName);
		}

		if (transformer != null || onlyLocal || superTemplate == null) {
			return transformer;
		}
		return (BasicNamedElement) ((TemplateDescription) superTemplate).getRMapElement(aName, TRANSFORMER_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getTransformerMap(boolean)
	 */
	public Iterator getTransformers(boolean onlyLocal) throws KilimException  {
		if (onlyLocal) {
			if (transformers == null) {
				return KILIM.EMPTY_ITERATOR;
			}
			return transformers.values().iterator();
		}
		
		LinkedHashMap map = new LinkedHashMap();
		addMapVisible(map, TRANSFORMER_KEY, false);
		return map.values().iterator();
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getInstance(String, boolean)
	 */
	public Instance getInstance(String aName, boolean onlyLocal) throws KilimException {
		LinkedHashMap map = getMap(INSTANCE_KEY, false);
		Instance instance = null;
		if (map != null) {
			instance = (Instance) map.get(aName);
		}
		
		if (instance != null || onlyLocal || superTemplate == null) {
			return instance;
		}
		return (Instance) ((TemplateDescription) superTemplate).getRMapElement(aName, INSTANCE_KEY);
	}
			
	/**
	 * @see org.objectweb.kilim.description.Template#getInstanceMap(boolean)
	 */
	public Iterator getInstances(boolean onlyLocal) throws KilimException {
		if (onlyLocal) {
			if (instances == null) {
				return KILIM.EMPTY_ITERATOR;
			}
			return instances.values().iterator();
		}
		
		LinkedHashMap map = new LinkedHashMap();
		addMapVisible(map, INSTANCE_KEY, false);
		
		Collection vls = map.values();
		return vls.iterator();
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#addProviderReference(ProviderReference)
	 */
	public void addProviderReference(Reference reference) throws KilimException {
		if (reference == null) {
			throw new KilimException("illegal null reference in addProviderReference of TemplateImpl " + name);
		}
		if (!reference.providesValue()) {
			throw new KilimException("illegal attempt to use a non provider reference in addProvider");
		}
		
		String rName = reference.getTargetName();
		if (rName == null) {
			throw new KilimException("illegal null name in addProvider of TemplateImpl " + name);
		}
		ReferenceData data = (ReferenceData) getReferenceMap().get(rName);
		
		if (data == null) {
			getReferenceMap().put(rName, new ReferenceData(PROVIDER_REFERENCE, reference));
			return;
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#removeProviderReference(String)
	 */
	public void removeProviderReference(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("illegal null name in removeProviderReference of TemplateImpl " + name);
		}

		ReferenceData data = (ReferenceData) getReferenceMap().get(aName);
		
		if (data == null) {
			throw new KilimException("name " + aName + " is unknown in removeProviderReference of TemplateImpl" + name);
		}
		
		if (data.referenceKind == PROVIDER_REFERENCE) {
			getReferenceMap().remove(aName);
		} else {
			throw new KilimException("use of a transformer name in removeProviderReference of TemplateImpl " + name);
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#getProviderReference(String, boolean)
	 */
	public Reference getProviderReference(String aName, boolean onlyLocal) throws KilimException {
		if (aName == null) {
			throw new KilimException("illegal null name in getProviderReference of TemplateImpl" + name);
		}

		ReferenceData data = (ReferenceData) getReferenceMap().get(aName);
				
		if (data == null) {
			//throw new KilimException("name " + aName + " is unknown in getProviderReference of TemplateImpl " + name);
			return null;
		}
		
		if (data.referenceKind == PROVIDER_REFERENCE) {
			return (Reference) data.reference;
		} else {
			throw new KilimException("use of a transformer name in removeProviderReference of TemplateImpl " + name);
		}
	}
		
	/**
	 * @see org.objectweb.kilim.description.Template#getTransformerReference(String, boolean)
	 */
	public Reference getTransformerReference(String aName, boolean onlyLocal) throws KilimException {
		if (aName == null) {
			throw new KilimException("illegal null name in getTransformerReference of TemplateImpl " + name);
		}

		ReferenceData data = (ReferenceData) getReferenceMap().get(aName);
		
		if (data == null) {
			//throw new KilimException("name " + aName + " is unknown in getTransformerReference of TemplateImpl " + name);
			return null;
		}
		
		if (data.referenceKind == TRANSFORMER_REFERENCE) {
			return (Reference) data.reference;
		} else {
			throw new KilimException("use of a transformer name in removeProviderReference of TemplateImpl " + name);
		}
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getUnresolvedProviderReferences()
	 */
	public Iterator getProviderReferences(boolean onlyLocal) {
		List list = new ArrayList();
		getReferences(list, onlyLocal, PROVIDER_REFERENCE);
		return list.listIterator();
	}

	/**
	 * @see org.objectweb.kilim.description.Template#addTransformerReference(TransformerReference)
	 */
	public void addTransformerReference(Reference reference) throws KilimException {
		if (reference == null) {
			throw new KilimException("illegal null reference in addProviderReference of TemplateImpl " + name);
		}
		if (!reference.performsAction()) {
			throw new KilimException("attempt to use a non transformer reference in addTransformer");
		}
		
		String rName = reference.getTargetName();
		if (rName == null) {
			throw new KilimException("illegal null name in addTransformer of TemplateImpl");
		}
		ReferenceData data = (ReferenceData) getReferenceMap().get(rName);
		
		if (data == null) {
			getReferenceMap().put(rName, new ReferenceData(TRANSFORMER_REFERENCE, reference));
			return;
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#removeProviderReference(String)
	 */
	public void removeTransformerReference(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("illegal null name in removeTransformerReference of TemplateImpl " + name);
		}

		ReferenceData data = (ReferenceData) getReferenceMap().get(aName);
		
		if (data == null) {
			throw new KilimException("transformer name " + aName + " is unknown in removeTransformerReference of TemplateImpl " + name);
		}
		
		if (data.referenceKind == TRANSFORMER_REFERENCE) {
			getReferenceMap().remove(aName);
		} else {
			throw new KilimException("use of a provider name in removeTransformerReference of TemplateImpl " + name);
		}
	}
		
	/**
	 * @see org.objectweb.kilim.description.Template#getUnresolvedTransformerReferences()
	 */
	public Iterator getTransformerReferences(boolean onlyLocal) {
		List list = new ArrayList();
		getReferences(list, onlyLocal, TRANSFORMER_REFERENCE);
		return list.listIterator();
	}	
		
	/**
	 * @see org.objectweb.kilim.description.Template#getReference(String, boolean)
	 */
	public Reference getReference(String aName, boolean onlyLocal) throws KilimException {
		if (aName == null) {
			throw new KilimException("illegal null name in getProviderReference of TemplateImpl" + name);
		}

		ReferenceData data = (ReferenceData) getReferenceMap().get(aName);
		
		if (data == null) {
			return null;
		}
		
		return (Reference) data.reference;
	}

	/**
	 * @see org.objectweb.kilim.description.Template#addLocalPort(Port)
	 */
	public void addLocalPort(Port port) throws KilimException {
		addLocalMapElement(port, PORT_KEY);
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#removeLocalPort(String)
	 */
	public void removeLocalPort(String aName) throws KilimException {
		removeLocalMapElement(aName, PORT_KEY);		
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#addLocalProperty(Property)
	 */
	public void addLocalProperty(Property property) throws KilimException {
		addLocalMapElement(property, PROPERTY_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#removeLocalProperty(String)
	 */
	public void removeLocalProperty(String aName) throws KilimException {
		removeLocalMapElement(aName, PROPERTY_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#addProvider(Provider)
	 */
	public void addLocalProvider(BasicNamedElement aProvider) throws KilimException {
		addLocalMapElement(aProvider, PROVIDER_KEY);
	}	

	/**
	 * @see org.objectweb.kilim.description.Template#removeLocalProvider(String)
	 */
	public void removeLocalProvider(String aName) throws KilimException {
		removeLocalMapElement(aName, PROVIDER_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#addLocalTransformer(Transformer)
	 */
	public void addLocalTransformer(BasicNamedElement transformer) throws KilimException {
		addLocalMapElement(transformer, TRANSFORMER_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#removeLocalTransformer(String)
	 */
	public void removeLocalTransformer(String aName) throws KilimException {
		removeLocalMapElement(aName, TRANSFORMER_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#addInstance(Instance)
	 */
	public void addLocalInstance(Instance aInstance) throws KilimException {
		addLocalMapElement(aInstance, INSTANCE_KEY);
	}	

	/**
	 * @see org.objectweb.kilim.description.Template#removeLocalInstance(String)
	 */
	public void removeLocalInstance(String aName) throws KilimException {
		removeLocalMapElement(aName, INSTANCE_KEY);
	}
	
	/**
	 * adds a slot declatation in the template description.
	 * @param aSlot : the slot to be added.
	 * @throws KilimException : generated when aSlot is null or has already been added previously.
	 */
	public void addLocalSlot(Slot aSlot) throws KilimException {
		addLocalMapElement(aSlot, SLOT_KEY);
	}
	
	/**
	 * removes a slot definition from a template description.
	 * @param aName : the name of the the slot to be removed.
	 * @throws KilimException : generated when aSlot is null or unknown.
	 */
	public void removeLocalSlot(String aName) throws KilimException {
		removeLocalMapElement(aName, SLOT_KEY);
	}

	/**
	 * @see org.objectweb.kilim.description.Template#addLocalTrigger(Trigger)
	 */
	public void addLocalTrigger(Trigger aTrigger) throws KilimException {
		addLocalListElement(aTrigger, TRIGGER_KEY);
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#removeLocalTrigger(Trigger)
	 */
	public void removeLocalTrigger(Trigger aTrigger) throws KilimException {
		removeLocalListElement(aTrigger, TRIGGER_KEY);
	}
		
	/**
	 * @see org.objectweb.kilim.description.Template#addBinding(Binding)
	 */
	public void addLocalBinding(Binding aBinding) throws KilimException {
		addLocalListElement(aBinding, BINDING_KEY);
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#removeBinding(String)
	 */
	public void removeLocalBinding(Binding aBinding) throws KilimException {
		removeLocalListElement(aBinding, BINDING_KEY);
	}
	
		/**
	 * @see org.objectweb.kilim.description.Template#addBinding(Binding)
	 */
	public void addLocalPlug(Plug aPlug) throws KilimException {
		addLocalListElement(aPlug, PLUG_KEY);
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#removeLocalPlug(String)
	 */
	public void removeLocalPlug(Plug aPlug) throws KilimException {
		removeLocalListElement(aPlug, PLUG_KEY);
	}
		
	/**
	 * @see org.objectweb.kilim.description.Template#getTrigger(String, boolean)
	 */
	public Iterator getTriggers(String aName, boolean onlyLocal) {
		if (onlyLocal) {
			return getLocalListElements(aName, TRIGGER_KEY);
		}
		
		List list = new ArrayList();
		addVisibleListElements(list, aName, TRIGGER_KEY);
		return list.listIterator();
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#getTrigger(String, boolean)
	 */
	public Iterator getBindings(String aName, boolean onlyLocal) {
		if (onlyLocal) {
			return getLocalListElements(aName, BINDING_KEY);
		}
		
		List list = new ArrayList();
		addVisibleListElements(list, aName, BINDING_KEY);
		return list.listIterator();
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getTrigger(String, boolean)
	 */
	public Iterator getPlugs(String aName, boolean onlyLocal) {
		if (onlyLocal) {
			return getLocalListElements(aName, PLUG_KEY);
		}
		
		List list = new ArrayList();
		addVisibleListElements(list, aName, PLUG_KEY);
		return list.listIterator();
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#getTriggerMap(boolean)
	 */
	public Iterator getTriggers(boolean onlyLocal) {
		if (onlyLocal) {
			return getLocalListElements(TRIGGER_KEY);
		}
					
		List list = new ArrayList();
		addAllVisibleListElements(list, TRIGGER_KEY);
		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
	
		return list.listIterator();
	}

	/**
	 * @see org.objectweb.kilim.description.Template#getBindings(boolean)
	 */
	public Iterator getBindings(boolean onlyLocal) {
		if (onlyLocal) {
			return getLocalListElements(BINDING_KEY);
		}

		List list = new ArrayList();
		addAllVisibleListElements(list, BINDING_KEY);
		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return list.listIterator();
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#getBindings(boolean)
	 */
	public Iterator getPlugs(boolean onlyLocal) {
		if (onlyLocal) {
			return getLocalListElements(PLUG_KEY);
		}

		List list = new ArrayList();
		addAllVisibleListElements(list, PLUG_KEY);
		if (list.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return list.listIterator();
	}

	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return name + " :: " + super.toString();
	}
	
	/**
	 * @see org.objectweb.kilim.description.Template#containsNewDefinitions()
	 */
	public boolean containsNewDefinitions() throws KilimException {
		if ((allProviders != null && allProviders.size() > 0)
		|| (transformers != null && transformers.size() > 0)
		|| (bindings != null && bindings.size() > 0)
		|| (triggers != null && triggers.size() > 0)
		|| (instances != null && instances.size() > 0)
		|| (slots != null && slots.size() > 0)
		|| (plugs != null && plugs.size() > 0)) {
			return true;
		}
		return false;
	}
		
	private Binding getRBinding(String aName) {
		Binding binding = null;
		
		if (bindings != null) {
			binding = (Binding) getMap(BINDING_KEY, true).get(aName);
		}
		
		if (binding != null || superTemplate == null) {
			return binding;
		}
		return ((TemplateDescription) superTemplate).getRBinding(aName);
	}	
	
	/**
	 * Method checkLocalNameUnicity.
	 * @param aName :
	 * @throws KilimException :
	 */
	private boolean localNameUnicity (String aName) throws KilimException {		
		if (getAnyProvider(aName, true) != null || getSlot(aName, true) != null
			|| getTransformer(aName, true) != null || getInstance(aName, true) != null) {
			return false;
		}
		return true;
	}
	
	private void checkOverride(String aName, int aKey, boolean canOverride) throws KilimException {
		int result = checkUpOverride(aName, aKey);
		if (result == ILLEGAL_OVERRIDE) {
			throw new KilimException("illegal override of element " + aName);
		}
			
		if (result == NAME_CLASH) {
			throw new KilimException("name clash for element (1) " + aName);
		}
		
		result = checkDownOverride(aName, aKey, canOverride);
		if (result == ILLEGAL_OVERRIDE) {
			throw new KilimException("illegal override of element " + aName);
		} 
		if (result == NAME_CLASH) {
			throw new KilimException("name clash for element (2) " + aName);
		}	
	}
	
	/**
	 * Method checkOverride : It checks whether override is legal or not.
	 * This method is not optimized and checks all the hierarchy in order to detect illegal situations.
	 * @param aName :
	 * @param tabKey :
	 * @throws KilimException :
	 */
	private int checkUpOverride(String aName, int tabKey) throws KilimException {
		//no super template. Override is thus trivially legal.
		if (superTemplate == null) {
			return LEGAL_OVERRIDE;
		}
		
		//aName is present in super template with a private status
		NamedElement element = ((TemplateDescription) superTemplate).getLocalMapElement(aName, tabKey);

		if (element != null) {
			if (element.getStatus() == KILIM.PRIVATE) {
				return ILLEGAL_OVERRIDE;
			}
		}
		
		//aName is present in super template in a different name space
		if ((tabKey != PROVIDER_KEY && tabKey != PORT_KEY && tabKey != PROPERTY_KEY 
			&& tabKey != ANY_PROVIDER_KEY && getLocalMapElement(aName, ANY_PROVIDER_KEY) != null)			
			|| (tabKey != TRANSFORMER_KEY && getLocalMapElement(aName, TRANSFORMER_KEY) != null)
			|| (tabKey != INSTANCE_KEY && getLocalMapElement(aName, INSTANCE_KEY) != null)
			|| (tabKey != SLOT_KEY && getLocalMapElement(aName, SLOT_KEY) != null)) {
			return NAME_CLASH;
		}
		
		//no illegal situation detected : check in super template ....
		return ((TemplateDescription) superTemplate).checkUpOverride(aName, tabKey);
	}
	
	private int checkDownOverride(String aName, int tabKey, boolean canOverride) {
		NamedElement element = getLocalMapElement(aName, tabKey);

		//aName is present in template overriding a private element
		if (element != null) {
			if (!canOverride) {
				return ILLEGAL_OVERRIDE;
			}	
		}
		
		//aName is present in template in a different name space
		if ((tabKey != PROVIDER_KEY && tabKey != PORT_KEY && tabKey != PROPERTY_KEY 
			&& tabKey != ANY_PROVIDER_KEY && getLocalMapElement(aName, ANY_PROVIDER_KEY) != null)			
			|| (tabKey != TRANSFORMER_KEY && getLocalMapElement(aName, TRANSFORMER_KEY) != null)
			|| (tabKey != INSTANCE_KEY && getLocalMapElement(aName, INSTANCE_KEY) != null)
			|| (tabKey != SLOT_KEY && getLocalMapElement(aName, SLOT_KEY) != null)) {
			return NAME_CLASH;
		}
		
		//no detected illegal situation detected : check in subTemplates ...
		Iterator iter = getSubTemplates();
		while (iter.hasNext()) {
			int result = 0;
			try {
			result = ((TemplateDescription) iter.next()).checkDownOverride(aName, tabKey, canOverride);
			} catch (ClassCastException e) {
				e.printStackTrace();
			}
			if (result == ILLEGAL_OVERRIDE || result == NAME_CLASH) {
				return result;
			}
		}
		return LEGAL_OVERRIDE;
	}
				
	/* ****************************************************************************************************
	 * The following methods are generic methods used to handle template elements stored in LinkedHashMap tables.
	 * ***************************************************************************************************/
	
	private String[] elementIdents = { 
		"", "provider", "transformer", "property", "instance", "port", "trigger", "binding", "slot", "plug"
	};
		
	//The addLocalElement method is a method for adding a named template element in its LinkedHashMap
	private void addLocalMapElement(NamedElement aElement, int tabKey) throws KilimException {
		if (aElement == null) {
			throw new KilimException("attempt to add a null " + elementIdents[tabKey] + " in template " + name);
		}
		String pName = aElement.getLocalName();
		if (pName == null) {
			throw new KilimException("attempt to add a " + elementIdents[tabKey] + " with a null name in template " + name);
		}

		if (!localNameUnicity(pName)) {
			throw new KilimException("name clash for " + pName + " in template " + name);
		}
		checkOverride(pName, tabKey, aElement.getStatus() != KILIM.PRIVATE);
		switch(tabKey) {
			case PROVIDER_KEY :
				BasicElement src = ((Provider) aElement).getSource();
				if (src instanceof InlinedElement) {
					((InlinedElement) src).setLocalName(pName);
				}	
			case PORT_KEY :
			case PROPERTY_KEY :
			case ANY_PROVIDER_KEY :
				getMap(tabKey, true).put(pName, new ProviderData(tabKey, aElement));
				break;
			case TRANSFORMER_KEY :
				BasicElement act = ((Transformer) aElement).getAction();
				if (act instanceof InlinedElement) {
					((InlinedElement) act).setLocalName(pName);
				}
			default :
				getMap(tabKey, true).put(pName, aElement);
				break;
		}
		aElement.setContainingTemplate(this);
	}
	
	//The removeLocalElement method is a method for removing a named template element from its LinkedHashMap
	private void removeLocalMapElement(String aName, int tabKey) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to remove a " + elementIdents[tabKey] + " through a null name in template " + name);
		}
		LinkedHashMap map = getMap(tabKey, false);
		if (map != null) {
			TemplateElementImpl element = (TemplateElementImpl) map.get(aName);
			if (map != null) {
				map.remove(aName);
			}
			element.setContainingTemplate(null);	
		}
	}
	
	/**
	 * Method addMapVisible.
	 * This method gets all locally declared elements of a given kind and add them to the 
	 * LinkedHashMap received as an argument.
	 * @param map : a hash map to be completed with the inherited elements.
	 * @param tabKey : a key to designate the kind of the elements to be handled.
	 * @param isParent : a boolean value indicating whether the current template is a parent template or not..
	 */
	private void addMapVisible(LinkedHashMap map, int tabKey, boolean isParent) throws KilimException {
		Iterator enum = null;
		switch(tabKey) {
			case PROVIDER_KEY :
			case PROPERTY_KEY :
			case PORT_KEY :
			case ANY_PROVIDER_KEY :
				if (allProviders != null) {
					enum = allProviders.values().iterator();
				}		
				break;
			case TRANSFORMER_KEY :
				if (transformers != null) {
					enum = transformers.values().iterator();
				}
				break;
			case INSTANCE_KEY :
				if (instances != null) {
					enum = instances.values().iterator();
				}
				break;
			case TRIGGER_KEY :
				if (triggers != null) {
					enum = triggers.iterator();
				}
				break;
			case SLOT_KEY :
				if (slots != null) {
					enum = slots.values().iterator();
				}
				break;
			default :
				throw new InternalException("illegal key " + tabKey + " in _addMapVisible");
		}
		
		NamedElement provider = null;
		if (enum != null) {
			while (enum.hasNext()) {
				Object element = enum.next();
				String eName;
				switch (tabKey) {
					case PROVIDER_KEY :
					case PROPERTY_KEY :
					case PORT_KEY :
					case ANY_PROVIDER_KEY :
						provider =  (BasicNamedElement) ((ProviderData) element).provider;				
						break;
					default :
						provider = (NamedElement) element;
						break;
				}
				
				eName = provider.getLocalName();
				//check whether the element is masked (i.e. it has been overriden in inherited templates)
				if (map.containsKey(eName)) {
					continue;
				}
			
				//check whether the element is visible in the inherited templates (i.e; it has a status 
				//which is different from PRIVATE).
				if ((provider.getStatus() != KILIM.PRIVATE) ||  !isParent) {
					map.put(eName, element);
				}
			}
		}
		
		if (superTemplate != null) {
			((TemplateDescription) superTemplate).addMapVisible(map, tabKey, true);
		}
	}

	//The getLocalElement method is a method for retrieving a named template element from 
	//its local LinkedHashMap
	private NamedElement getLocalMapElement(String aName, int tabKey) {
		switch(tabKey) {
			case PROVIDER_KEY :
			case PROPERTY_KEY :
			case PORT_KEY :
			case ANY_PROVIDER_KEY :
				if (allProviders == null) {
					return null;
				}
				ProviderData data = (ProviderData) allProviders.get(aName);
				if (data == null) {
					return null;
				}		
				return (NamedElement) data.provider;
			case TRANSFORMER_KEY :
				if (transformers == null) {
					return null;
				} else {			
					return (NamedElement) transformers.get(aName);
				}
			case INSTANCE_KEY :
				if (instances == null) {
					return null;
				} else {			
					return (NamedElement) instances.get(aName);
				}
			case SLOT_KEY :
				if (slots == null) {
					return null;
				} else {
					return (NamedElement) slots.get(aName);
				}
			default :
				throw new InternalException("illegal key " + tabKey + " in getRElement for name " + aName);
		}
	}
	
	//The getRMapElement method is a recursive method for retrieving a named template element 
	//stored in a LinkedHashMap
	private NamedElement getRMapElement(String aName, int tabKey) throws KilimException {
		NamedElement element = getLocalMapElement(aName, tabKey);

		if (element != null || superTemplate == null) {
			return element;
		}

		return getSuperTemplate().getRMapElement(aName, tabKey);
	}
	
	//This method returns the map corresponding to a given element kind. The map is created 
	//if it does not exist and if the value of the create argument is equal to true.
	private LinkedHashMap getMap(int tabKey, boolean create) {
		switch(tabKey) {
			case PORT_KEY :
			case PROPERTY_KEY :
			case PROVIDER_KEY :
			case ANY_PROVIDER_KEY :
				if (allProviders == null && create) {
					allProviders = new LinkedHashMap();
				}
				return allProviders;
			case TRANSFORMER_KEY :
				if (transformers == null && create) {
					transformers = new LinkedHashMap();
				}
				return transformers;
			case INSTANCE_KEY :
				if (instances == null && create) {
					instances = new LinkedHashMap();
				}
				return instances;
			case SLOT_KEY :
				if (slots == null && create) {
					slots = new LinkedHashMap();
				}
				return slots;
			default :
				throw new InternalException("illegal key " + tabKey + " in getMap");
		}			
	}
	
	/* ****************************************************************************************************
	 * The following methods are generic methods used to handle template elements references.
	 * ***************************************************************************************************/
	
	private void getReferences(List aList, boolean onlyLocal, int aKind) {
		if (references != null) {
			Iterator iter = references.values().iterator();
					
			while (iter.hasNext()) {
				ReferenceData data = (ReferenceData) iter.next();
				if (data.referenceKind == aKind) {
					aList.add(data.reference);
				}		
			}
		}
		if (superTemplate == null || onlyLocal) {
			return;
		}
		((TemplateDescription) superTemplate).getReferences(aList, onlyLocal, aKind);
	}
	
	private LinkedHashMap getReferenceMap() {
		if (references == null) {
			references = new LinkedHashMap();
		}
		return references;
	}
	
	/* ****************************************************************************************************
	 * The following methods are generic methods used to handle template elements stored in linked lists.
	 * ***************************************************************************************************/
		
	//The addLocalElement method is a method for adding a named template element in its LinkedHashMap
	private void addLocalListElement(TemplateElementImpl aElement, int listKey) throws KilimException {
		if (aElement == null) {
			throw new KilimException("attempt to add a null " + elementIdents[listKey] + " in template " + name);
		}
		getList(listKey, true).add(aElement);
		aElement.setContainingTemplate(this);
	}
	
	//The removeLocalElement method is a method for removing a named template element from its LinkedHashMap
	private void removeLocalListElement(TemplateElementImpl aElement, int listKey) throws KilimException {
		if (aElement == null) {
			throw new KilimException("attempt to remove a " + elementIdents[listKey] + " through a null reference in template " + name);
		}
		List list = getList(listKey, false);
		if (list == null) {
			throw new KilimException("attempt to remove a " + elementIdents[listKey] + " from an empty template " + name);
		}
		
		list.remove(aElement);
		aElement.setContainingTemplate(null);	
	}
	
	//This method returns the list used to store a template element.
	private List getList(int tabKey, boolean create) {
		switch(tabKey) {
			case BINDING_KEY :
				if (bindings == null && create) {
					bindings = new ArrayList();
				}
				return bindings;
			case PLUG_KEY :
				if (plugs == null && create) {
					plugs = new ArrayList();
				}
				return plugs;
			case TRIGGER_KEY :
				if (triggers == null && create) {
					triggers = new ArrayList();
				}
				return triggers;
		}
		return null;				
	}
	
	private Iterator getLocalListElements(int listKey) {	
		List list = getList(listKey, false);
		if (list == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		
		Iterator iter = list.listIterator();
		List result = new ArrayList();
		
		Object element = null;
		while (iter.hasNext()) {
			switch (listKey) {
				case BINDING_KEY :
					element =  iter.next();
					break;
				case TRIGGER_KEY :
					element = iter.next();
				case PLUG_KEY :
					element = iter.next();
					break;
			}
			result.add(element);
		}
		if (result.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return result.listIterator();
	}
	
	private Iterator getLocalListElements(String aName, int listKey) {	
		
		List list = getList(listKey, false);
		if (list == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		
		Iterator iter = list.listIterator();
		List result = new ArrayList();
		
		Object element = null;
		String nm = null;
		while (iter.hasNext()) {
			switch (listKey) {
				case BINDING_KEY :
					element =  iter.next();
					nm = ((Binding) element).getPortName();
					break;
				case TRIGGER_KEY :
					element = iter.next();
					nm = ((Trigger) element).getSourceName();
					break;
				case PLUG_KEY :
					element = iter.next();
					nm = ((Plug) element).getSlotName();
					break;
			}
			if (aName.equals(nm)) {
				result.add(element);
			}
		}
		if (result.size() == 0) {
			return KILIM.EMPTY_ITERATOR;
		}
		return result.listIterator();
	}
	
	private void addVisibleListElements(List list, String aName, int listKey) {
		boolean stop = false;

		//look up should stop as soon as a port, property or provider definition has been found , since this override maskes all previous binding
		//and trigger definitions ... It should be noted here that there is no restriction on the override process.
		if ((listKey == BINDING_KEY) || (listKey == TRIGGER_KEY)) {
			if (getLocalMapElement(aName, ANY_PROVIDER_KEY) != null) {
				stop = true;
			}
		}
		
		if (superTemplate != null && !stop) {
			((TemplateDescription) superTemplate).addVisibleListElements(list, aName, listKey);
		}
		
		List list1 = getList(listKey, false);
		if (list1 != null) {
			Iterator iter = list1.listIterator();
			Object element = null;
			String nm = null;
			while (iter.hasNext()) {
				TemplateElementImpl elem = null;
				switch (listKey) {
					case BINDING_KEY :
						element =  iter.next();
						nm = ((Binding) element).getPortName();
						break;
					case TRIGGER_KEY :
						element = iter.next();
						nm = ((Trigger) element).getSourceName();
						break;
					case PLUG_KEY :
						element = iter.next();
						nm = ((Plug) element).getSlotName();
						break;
				}
				if (aName.equals(nm)) {
					list.add(element);
				}
			}
		}		
	}
	
	private void addAllVisiblePlugs(List list) {
		if (superTemplate != null) {
			((TemplateDescription) superTemplate).addAllVisiblePlugs(list);
		}

		List list1 = getList(PLUG_KEY, false);
		if (list1 != null) {
			Iterator iter = list1.listIterator();
			TemplateElementImpl element;
			loop : while (iter.hasNext()) {
				element = (TemplateElementImpl) iter.next();
				list.add(element);
			}
		}
	}
	
	private void addAllVisibleListElements(List list, int listKey) {
		if (listKey == PLUG_KEY) {
			addAllVisiblePlugs(list);
			return;
		}
		
		LinkedHashMap allVisible = getProviderLastDefinitionTable();
		addAllVisibleListElements1(list, listKey, 0, allVisible);
	}
	
	private void addAllVisibleListElements1(List list, int listKey, int aLevel, LinkedHashMap aDefTable) {		
		if (superTemplate != null) {
			((TemplateDescription) superTemplate).addAllVisibleListElements1(list, listKey, aLevel + 1, aDefTable);
		}

		List list1 = getList(listKey, false);
		if (list1 != null) {
			Iterator iter = list1.listIterator();		
			while (iter.hasNext()) {
				Object element = iter.next();
				String nm = null;
				if (listKey == BINDING_KEY) {
					nm = ((Binding) element).getPortName();
				} else if (listKey == TRIGGER_KEY) {
					nm = ((Trigger) element).getSourceName();
				}
				
				if ("THIS".equals(nm)) {
					list.add(element);
					continue;
				}
				
				Integer level1 = (Integer) aDefTable.get(nm);
				if (level1 == null) {
					throw new InternalException("Problem in AddAllVisibleListElements1 of TemplateDescription : unknown element " + nm);
				} else {
					if (aLevel <= level1.intValue()) {
						list.add(element);
					}
				}
			}
		}
	}
	
	LinkedHashMap definitionTable = new LinkedHashMap();

	private LinkedHashMap getProviderLastDefinitionTable() {
		//LinkedHashMap definitionTable = new LinkedHashMap();
		definitionTable.clear();
		getPLastDefinitionSet(definitionTable, 0);
		return definitionTable;
	}
	
	private void getPLastDefinitionSet(LinkedHashMap aTable, int aLevel) {
		Integer level = new Integer(aLevel);
		if (allProviders != null) {
			Iterator iter = allProviders.keySet().iterator();
			while (iter.hasNext()) {
				String name = (String) iter.next();
				if (aTable.containsKey(name)) {
					continue;
				}
				ProviderData data = (ProviderData) allProviders.get(name);
				if (data.provider instanceof InlinedElement) {
					continue;
				}
				aTable.put(name, level);
			}
		}
		
		if (slots != null) {
			Iterator iter = slots.values().iterator();
			while (iter.hasNext()) {
				Slot slot = (Slot) iter.next();
				String slotName = slot.getLocalName();
				Iterator iter1 = slot.getPorts();
				while (iter1.hasNext()) {
					Port port = (Port) iter1.next();
					String name = slotName + "/" + port.getLocalName();
					aTable.put(name, level);
				}
			}	
		}
				
		TemplateDescription superTmp = getSuperTemplate();		
		
		if (superTmp != null) {
			superTmp.getPLastDefinitionSet(aTable, aLevel + 1);
		}
	}
	
	/**
	 * Method getAllInterfaces.
	 * @param onlyLocal : 
	 * @return Iterator
	 */
	public Iterator getAllInterfaces(boolean onlyLocal) {
		LinkedHashMap allInterfaces = new LinkedHashMap();
		getAllInterfaces1(allInterfaces, onlyLocal);
		return allInterfaces.values().iterator();
	}
	
	private void getAllInterfaces1(LinkedHashMap aTable, boolean onlyLocal) {
		if (allProviders != null) {
			Iterator iter = allProviders.keySet().iterator();
			while (iter.hasNext()) {
				String name = (String) iter.next();
				if (aTable.containsKey(name)) {
					continue;
				}
				ProviderData data = (ProviderData) allProviders.get(name);
				if (data.provider instanceof InlinedElement) {
					continue;
				}
				aTable.put(name, data.provider);
			}
		}
				
		TemplateDescription superTmp = getSuperTemplate();		
		
		if (superTmp != null && !onlyLocal) {
			superTmp.getAllInterfaces1(aTable, onlyLocal);
		}
	}
	
	/**
	 * Returns the Resource Loader that was used to load this <code>TemplateDescription</code>.
	 * @return the Resource Loader that was used to load this <code>TemplateDescription</code>.
	 */
	public ResourceLoader getResourceLoader() {
		return resourceLoader;
	}

	/**
	 * Sets the Resource Loader that was used to load this <code>TemplateDescription</code>.
	 * @param resourceLoader The resourceloader that was used to load this <code>TemplateDescription</code>.
	 */
	public void setResourceLoader(ResourceLoader resourceLoader) {
		this.resourceLoader = resourceLoader;
	}
}