package org.objectweb.kilim.description;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 */
public class Transformer extends BasicNamedElementImpl {
	private BasicElement action;
	
	/**
	 * A public constructor to transformers.
	 * @param aName : the name associated to the transformer
	 * @param aStatus : the status of the transformer. It should be one of KILIM.PUBLIC, KILIM.PROTECTED, KILIM.PRIVATE.
	 * @param isP : is true if the transformer can be used a a provider.
	 * @param aTemplate : the template in which the transformer has been declared.
	 * @throws KilimException : generated if aName or aTemplate is null, if aStatus has an illegal value.
	 */
	public Transformer(String aName, int aStatus, boolean isP, TemplateDescription aTemplate) throws KilimException {
		super(aName, aStatus, isP, true, aTemplate);
		action = new NullElement(aName, aStatus, isP, true, aTemplate);
	}
	
	/**
	 * A public constructor to transformers (that cannot be used as a provider). It is strictly equivalent to (aName, aStatus, false, true, aTemplate, aTemplate);
	 * @param aName : the name associated to the transformer
	 * @param aStatus : the status of the transformer. It should be one of KILIM.PUBLIC, KILIM.PROTECTED, KILIM.PRIVATE.
	 * @param aTemplate : the template in which the transformer has been declared.
	 * @throws KilimException : generated if aName or aTemplate is null, if aStatus has an illegal value.
	 */
	public Transformer(String aName, int aStatus, TemplateDescription aTemplate) throws KilimException {
		super(aName, aStatus, false, true, aTemplate);
	}
		
	/**
	 * @see org.objectweb.kilim.description.BasicElement#getKind()
	 */
	public int getKind() {
		return KILIM.TRANSFORMER;
	}

	/**
	 * sets the action (i.e. the effective mechanism represented by the transformer).
	 * @param aAction : the action to be performed by the transformer.
	 * @throws KilimException : generated if aAction is null or if aAction has not been created as an element able to perform action.
	 */
	public void setAction(BasicElement aAction) throws KilimException {
		if (aAction != null && !aAction.performsAction()) {
			throw new KilimException("illegal attempt to set a non transformer inline to the transformer " + getLocalName() + " in template " + getContainingTemplate().getName());
		}
		action = aAction;
	}
	
	/**
	 * returns the action (i.e. the effective mechanism represented by the transformer).
	 * @return InlinedElement. The action associated to the transformer.
	 * @throws KilimException : it is never generated.
	 */
	public BasicElement getAction() throws KilimException {
		return action;
	}
	
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setLocalName(String)
	 */
	public void setLocalName(String aName) throws KilimException {
		String oName = getLocalName();
		super.setLocalName(aName);
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalTransformer(oName);
			template.addLocalTransformer(this);
		}
	}
		
	/**
	 * @see org.objectweb.kilim.description.TemplateElement#setStatus(int)
	 */
	public void setStatus(int aStatus) throws KilimException {
		super.setStatus(aStatus);
		TemplateDescription template = getContainingTemplate();
		if (template != null) {
			template.removeLocalTransformer(getLocalName());
			template.addLocalTransformer(this);
		}
	}
}
