/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.model.instanciation.InstanciationMger;

/**
 * @author horn
 */
public interface Component extends ContainerElement  {				
	/**
	 * returns the template describing the component.
	 * @return TemplateDescription
	 */
	TemplateDescription getTemplate() ;
	
	/**
	 * returns a subcomponent identified by its local name. This methods does not recurse and therefore is not
	 * usable to find a subsubcomponent ....
	 * @param aName : the local name of a subcomponent.
	 * @return ComponentElement.
	 */
	Component getSubComponent(String aName);
	
	/**
	 * returns as an iterator the "direct" subcomponents of a component : this method does not recurse and thus only returns
	 * the subcomponent directly defined in the component.
	 * @return Iterator
	 */
	Iterator getSubComponents();

	/**
	 * adds a new subcomponent.
	 * @param aElement : the component to be added as a subcomponent.
	 * @throws KilimException : generated if aElement is null or is already a subcomponent.
	 */
	void addSubComponent(Component aElement) throws KilimException; 
	
	/**
	 * removes a "direct" subcomponent : this method only looks for subcomponent directly defined in the component.
	 * @param aLocalName : the local name of the subcomponent to be removed. 
	 * This method does nothing more than just  "unregistering" AElement as a subcomponent
	 * @throws KilimException : generated if aElement is null, or if the aElement is not a direct subcomponent
	 */
	void removeSubComponent(String aLocalName) throws KilimException;
	
	/**
	 * removes a "direct" subcomponent : this method only looks for subcomponent directly defined in the component.
	 * @param aLocalChild : the subcomponent to be removed. 
	 * This method does nothing more than just  "unregistering" AElement as a subcomponent
	 * @throws KilimException : generated if aElement is null, or if the aElement is not a direct subcomponent
	 */
	void removeSubComponent(Component aLocalChild) throws KilimException;

	/**
	 * Method getSlot returns a component slot identified by its local name.
	 * @param aLocalName : the local name of the slot
	 * @return Slot
	 * @throws KilimException :
	 */
	ComponentSlot getSlot(String aLocalName)  throws KilimException;
	
	/**
	 * returns as an iterator the slots defined in the component. Each element is thus an implementation of
	 * the ComponentSlot interface and an instance of the RtComponentSlot.
	 * @return Iterator : every member provided by the iterator implements the ComponentSlot interface and more precisly is an
	 * instance of the RtComponentSlot class.
	 */
	Iterator getSlots();
	
	/**
	 * returns as an iterator the slots the component is plugged into. The type of each element is thus a subtype of  ComponentSlot.
	 * @return Iterator: every member provided by the iterator implements the Component interface and more precisly is an
	 * instance of the RtComponent class.
	 */
	Iterator getPlugTos();
	
	/**
	 * plugs a component into a slot.
	 * @param aName : name of the  slot.
	 * @param aComponent : the component to be plugged in the slot.
	 * @throws KilimException : generated if aName or aComponent is null, if aName does not identifiy a component slot.
	 */
	void plug(String aName, Component aComponent) throws KilimException;
	
	/**
	 * unplugs a component form a slot.
	 * @param aName : name of the  slot.
	 * @param aComponent : the component to be unplugged from the slot .
	 * @throws KilimException : generated if aName or aComponent is null, if aName does not identifiy a component slot or if aCompoonent is not plugged in the slot.
	 */
	void unplug(String aName, Component aComponent) throws KilimException;
			
	/**
	 * creates a copy of a component as defined in the template (same elements, same plugs, ....).
	 * It does not perform any operations executed at runtime on the component on which the fork is invoked.
	 * @return Component
	 * @throws KilimException : 
	 */
	Component fork() throws KilimException;
	
	/**
	 * releases a component by (1) removing it as a subcomponent of its containing
	 * component, (2) unpluging it from all slots it is plugged in, (3) removing its naming context from parent context and factory
	 * from parent factory. This method however does not unbind DIRECTLY bound ports (i.e. bound through a bindValue or a
	 * bindProvider method).
	 * @throws KilimException : 
	 */
	void release() throws KilimException;
	
	/**
	 * sets the instanciation manager from the instanciation strategy defined in the class KilimConfiguration {@link org.objectweb.kilim.KilimConfiguration}
	 * @return InstanciationMgerI
	 * @throws KilimException :
	 */
	InstanciationMger getInstanciationMgerFromConfiguration() throws KilimException; 
	
	/**
	 * sets the instanciation manager to be used for the component. It overrides the
	 * manager defined in the instanciation strategy.
	 * @param aMger : the instanciation manager to be used
	 * @throws KilimException :
	 */
	void setInstanciationMger(InstanciationMger aMger) throws KilimException; 
	
	/**
	 * Method getInstanciationMger returns the instanciation manager to be used the component.
	 * @return InstanciationMger
	 */
	InstanciationMger getInstanciationMger();
	
	/**
	 * adds a controller
	 * @param aName : the name of the controller.
	 * @param aController : the controller to be added.
	 * @throws KilimException : generated if aName is null.
	 */
	void addController(String aName, Object aController) throws KilimException;
	
	/**
	 * removes a controller.
	 * @param aName : the name of the controller to be removed.
	 * @throws KilimException : generated if aName is null.
	 */
	void removeController(String aName) throws KilimException;
	
	/**
	 * returns a controller identified by its name.
	 * @param aName : the name of the controller.
	 * @return Object
	 * @throws KilimException : generated if aName is null.
	 */
	Object getController(String aName) throws KilimException;
	
	/**
	 * returns as an iterator the controllersz associated to a component.
	 * @return Iterator
	 */
	Iterator getControllers();
}