/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * A ComponentInterface is a runtime descriptor of an interface. 
 * This class provides therefore several methods to determine the real nature of the interface (unary port, nary port,  provider or property).
 * It supports morover several methods to bind or unbind the interface to values or value sources.
 */
	
public interface ComponentInterface extends ComponentSource {		
	/**
	 * Method isSingleValueInterface returns true if the interface is a single value port. It returns flase otherwise.
	 * @return boolean
	 */
	boolean isSingleValuePort();
	
	/**
	 * returns true if the interface is a collection port. It returns false otherwise.
	 * @return boolean
	 */
	boolean isCollectionPort();
	
	/**
	 * returns true if the interface is a property. It returns false otherwise.
	 * @return boolean
	 */
	boolean isProperty();
	
	/**
	 * returns true if the interface is a provider (i.e. an unbuffered value source).
	 * @return boolean
	 */
	boolean isProvider();
	
	/**
	 * binds an interface to a value source. This method has a behaviour which depends on the exact kind
	 * @param aProvider : the value source.
	 * @param jReplace : bindProvider applied to a single value port having already a provider generates a warning if jReplace is false.
	 * It silently replaces the existing value if jReplace is true.
	 * @throws KilimException : generated if aProvider is null.
	 */
	void bindProvider(RuntimeSource aProvider, boolean jReplace) throws KilimException;
	
	/**
	 * unbinds an interface and source value.
	 * @param aProvider : the source value to be unbound.
	 * @throws KilimException : generated if aProvider is null or is not bound to the interface.
	 */
	void unbindProvider(RuntimeSource aProvider) throws KilimException;	

	/**
	 * Method isModified.
	 * @return boolean
	 * @throws KilimException :
	 */
	boolean isModified()  throws KilimException;
		
	/**
	 * assigns a value to an interface. If a value has already been associated to the interface,
	 * "unbind" triggers are  executed prior to the new value assignment. Bind triggers are then executed.
	 * @param aValue : the value to be assigned to the interface.
	 * @throws KilimException : generated when applied to a provider
	 */
	void bindValue(Object aValue) throws KilimException;
	
	/**
	 * sets the value of an interface to null. Unbind triggers are executed (using the value stored in the interface prior to the unbind operation).
	 * @throws KilimException :
	 */
	void unbindValue() throws KilimException;
	
	/**
	 * reexecutes all triggers associated to the interface.
	 * @throws KilimException : 
	 */
	void update() throws KilimException;
}