/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.Iterator;

import org.objectweb.kilim.KilimException;

/**
 * @author horn
 * ContainerElement is an intermediate interface corresponding to component entities that can contain elements i.e. components and slots.
 * All containier elements have a state that is either "non-initialized" or "initialized". This state has been introduced because instanciation 
 * rises difficult and subtile problems which should be solved through precisely defined instanciation stategies. These strategies are 
 * usually enforced with the help of instanciation managers which should be executed prior to any access to a value. A container is thus 
 * created in the "non initialized" state. Access to the value of an element is only possible on an instanciated element which in turn requires 
 * that the container is instanciated. More precisely the evaluation algorithm of component interfaces always starts by looking whether the 
 * interface containier is initialized. The general instanciation strategy is invoked if it is not the case. It is the responsability of the instanciation 
 * manager to set the container in the "initialized" state.
 * 
 * A container is a naming context .
 */
public interface ContainerElement extends ComponentElement {
	/**
	 * returns true when the container is in the "initialized" state or not.
	 * @return boolean
	 */
	boolean isInitialized();

	/**
	 * sets the container in the"initialized" state.
	 */
	void setInitialized();

	/**
	 * returns the factory used to create the container.
	 * The result is a CompoonentFactory if the container is a component or a SlotFactory if the container is a slot.
	 * @return Factory
	 */
	Factory getFactory();
	
	/**
	 * returns true if the container is a component.
	 * @return boolean
	 */
	boolean isComponent();
	
	/**
	 * returns true if the container is a slot.
	 * @return boolean
	 */
	boolean isSlot();
		
	/**
	 * returns as an iterator the interfaces declared in the container.
	 * @return Iterator
	 */
	 Iterator getInterfaces();
	
	/**
	 * returns the local interface identified by its local name.
	 * @param aName : the local name of the interface.
	 * @return ComponentInterface : the reference of the interface.
	 * @throws KilimException : generated if aName is null or if no interface exists with the specified name.
	 */
	 ComponentInterface getInterface(String aName) throws KilimException;
	 
	 /**
	 * adds an Interface.in the slot.
	 * @param aInterface : the interface to be added.
	 * @throws KilimException : generated if aInterface is null or is already present in the slot.
	 */
	void addInterface(ComponentInterface aInterface) throws KilimException;
	
		/**
	 * removes the interface from a component.
	 * @param aElement : the interface to be removed
	 * @throws KilimException : generated if aElement is null, if the component has no interface or if the interface is not defined in the component
	 */
	void removeInterface(ComponentInterface aElement) throws KilimException;
}