/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Stack;
import java.util.HashMap;

import org.objectweb.kilim.KilimConfiguration;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.Instance;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NamedElement;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.description.Trigger;
import org.objectweb.kilim.model.instanciation.InstanciationMger;

/**
 * @author horn
 */
public class RtComponent extends RtComponentElement implements Component, RuntimeSource {

	private boolean isInitialized;
	private boolean isForkedComponent;
	private ComponentFactory factory;
	private TemplateDescription template;
	private TemplateDescription containingTemplate;
	private InstanciationMger instanciationMger;
	
	private List subComponents;
	private List interfaces;
	private List slots;
	private List plugTos;
	private List nAryListeners;
	
	//private List controllers;
	private HashMap controllers;

	/**
	 * The public constructor for RtComponents.
	 * @param aInstance : the template element corresponding to an instance
	 * @param aComponent : the containing component
	 * @param aFactory : the factoy used to create the component.
	 * @throws KilimException : generated when aInstance is null.
	 */
	public RtComponent(Instance aInstance, Component aComponent, ComponentFactory aFactory) throws KilimException {
		super(aInstance, aComponent);
		if (aInstance == null) {
			throw new KilimException("attempt to construct a component without instance description ");
		}
		if (aComponent != null) {
			aComponent.addSubComponent(this);
		}
		factory = aFactory;
		nbComponent++;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ContainerElement#isComponent()
	 */
	public boolean isComponent() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ContainerElement#isSlot()
	 */
	public boolean isSlot() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		NamedElement elem = (NamedElement) getElementDescription();
		return elem.getLocalName();
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#checkValue()
	 */
	public boolean hasValue() throws KilimException {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#checkValue(Stack)
	 */
	public boolean checkValue(Stack exclude) throws KilimException {
		return true;
	}

	/**
	 * adds a new interface in the component.
	 * @param aElement : the interface
	 * @throws KilimException : generated if aElement is null or if the local name of aElement clashes with the local name of another interface.
	 */
	public void addInterface(ComponentInterface aElement) throws KilimException {
		if (aElement == null) {				
			throw new KilimException("attempt to add a null interface to component " + getQualifiedName());
		}
		
		if (!(aElement instanceof RtComponentInterface)) {
			throw new KilimException("attempt to add a non Kilim interface to component " + getQualifiedName());
		}
				
		String lName = aElement.getLocalName();
		
		if (interfaces == null) {
			interfaces = new ArrayList();
		}
		
		if (containsElement(interfaces, lName)) {
			String interfSort = null;
			if (aElement.isProperty()) {
				interfSort = "property";
			} else if (aElement.isCollectionPort()) {
				interfSort = "collection interface ";
			} else if (aElement.isSingleValuePort()) {
				interfSort = "interface";
			} else {
				interfSort = "provider [" + aElement.getClass() + "]";
			}
			throw new KilimException("name clash when adding the " + interfSort + "  " + lName + " in component " + getQualifiedName());
		}
		
		interfaces.add(aElement);
	}
	
	/**
	 * removes the interface from a component.
	 * @param aElement : the interface to be removed
	 * @throws KilimException : generated if aElement is null, if the component has no interface or if the interface is not defined in the component
	 */
	public void removeInterface(ComponentInterface aElement) throws KilimException {
		if (aElement == null) {
			throw new KilimException("attempt to remove a null interface from component " + getQualifiedName());
		}
		
		if (interfaces == null) {
			throw new KilimException("no interface in current component");
		}
		
		Object previous = removeElement(interfaces, aElement.getLocalName());
		if (previous == null) {	
			throw new KilimException("interface " + aElement.getLocalName() + " unknown in current component " + getQualifiedName());
		}		
	}
	
	/**
	 * returns an interface defined in the component
	 * @param aName : the local name of the interface.
	 * @return ComponentInterface
	 */
	public ComponentInterface getInterface(String aName) {
		if (interfaces == null) {
			return null;
		}
		return (ComponentInterface) getElement(interfaces, aName);
	}
	
	/**
	 * returns as an iterator the interfaces defined in the component.
	 * @return Iterator
	 */
	public Iterator getInterfaces() {
		if (interfaces == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return interfaces.iterator();
	}
		
	/**
	 * @see org.objectweb.kilim.model.ContainerElement#getFactory()
	 */
	public Factory getFactory() {
		return factory;
	}
	
	/**
	 * @see org.objectweb.kilim.description.Instance#getTemplate()
	 */
	public TemplateDescription getTemplate() {
		Instance instance = ((Instance) getElementDescription());
		if (instance == null) {
			return null;
		} else {
			return instance.getTemplate();
		}
	}
	
	/**
	 * @see org.objectweb.kilim.description.Instance#setTemplate(Template)
	 */
	public void setTemplate(TemplateDescription aTemplate) {
		template = aTemplate;
	}	
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return getQualifiedName();
	}
	
	/**
	 * adds a component as a subcomponent of the current component.
	 * @param aElement : the subcomponent.
	 * @throws KilimException : generated if aElement is null or if it is already defined as a subcomponent in the component.
	 */
	public void addSubComponent(Component aElement) throws KilimException {
		if (aElement == null) {
			throw new KilimException("attempt to add a null subcomponent to component " + getQualifiedName());
		}
				
		String lName = aElement.getLocalName();
		
		if (subComponents == null) {
			subComponents = new ArrayList();
		}
		
		if (containsElement(subComponents, lName)) {
			throw new KilimException("name clash when adding the component " + lName + "in component " + getQualifiedName());
		}
		
		subComponents.add(aElement);
	}
	
	/**
	 * removes the subcomponent from the current component.
	 * @param aName : the name of the child component to be removed.
	 * @throws KilimException : generated if aName is null or is unknown (i.e. it is not a the name of a child component)
	 */
	public void removeSubComponent(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to remove a child component through a null name in component " + getQualifiedName());
		}
		
		if (subComponents == null) {
			throw new KilimException("attempt to remove a component from component " + getQualifiedName() + " without subcomponent ");
		}
		
		Object previous = removeElement(subComponents, aName);
		if (previous == null) {	
			throw new KilimException("attempt to remove unknown subcomponent " + aName + " from component " + getQualifiedName());
		}		
	}
	
	/**
	 * removes a component from the current component.
	 * @param aElement : the component to be removed.
	 * @throws KilimException : generated if aElement is null, if the component does not contain any subcomponent, 
	 * if aElement is not a subcomponent of the current component.
	 */
	public void removeSubComponent(Component aElement) throws KilimException {
		if (aElement == null) {
			throw new KilimException("attempt to remove a null component in component " + getQualifiedName());
		}
		
		if (subComponents == null) {
			throw new KilimException("attempt to remove a component from component " + getQualifiedName() + " without subcomponent ");
		}
		
		Object previous = removeElement(subComponents, aElement.getLocalName());
		if (previous == null) {	
			throw new KilimException("attempt to remove unknown subcomponent " + aElement.getLocalName() + " from component " + getQualifiedName());
		}		
	}
	
	/**
	 * returns a subcomponent  from its local name This method is not recursive and only looks for direct sub component.
	 * @param aName : the local name of the subcomponent.
	 * @return Component
	 */
	public Component getSubComponent(String aName) {
		if (subComponents == null) {
			return null;
		}
		return (Component) getElement(subComponents, aName);
	}
	
	/**
	 * returns as an iterator the direct subcomponents (i.e. the sub component of the current component).
	 * @return Iterator
	 */
	public Iterator getSubComponents() {
		if (subComponents == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return subComponents.iterator();
	}
	
	/**
	 * adds a new slot to a component.
	 * @param aSlot : the slot to be added
	 * @throws KilimException : generated if aSlot is null.
	 */
	public void addSlot(ComponentSlot aSlot) throws KilimException {
		if (aSlot == null) {
			throw new KilimException("attempt to add a null slot in component " + getQualifiedName());
		}
		if (slots == null) {
			slots = new ArrayList();
		}
		slots.add(aSlot);
	}

	/**
	 * removes a slot from a component.
	 * @param aName : the name of the slot to be removed.
	 * @throws KilimException : generated if aName is null, if the component does not contain slots or 
	 * if it does not contain a slot whose name is aName
	 */
	public void removeSlot(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to remove a slot through a null name in component " + getQualifiedName());
		}
		if (slots == null) {
			throw new KilimException("attempt to remove a slot from component " + getQualifiedName() + " without slot");
		}
		
		Object result = removeElement(slots, aName);
		if (result == null) {
			throw new KilimException("attempt to remove an unknown slot " + aName + "  from component " + getQualifiedName());
		}
	}
	
	/**
	 * returns a slot from its local name.
	 * @param aName : the local name of the slot.
	 * @return Slot
	 * @throws KilimException : generated if aName is null, if the component does not contain slots or 
	 * if it does not contain a slot whose name is aName
	 */
	public ComponentSlot getSlot(String aName)  throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get a null slot in component " + getQualifiedName());
		}
		if (slots == null) {			
			throw new KilimException("attempt to get slot " + aName + " from a component " + getQualifiedName() + " without slot");
		}

		RtComponentSlot slot1 = (RtComponentSlot) getElement(slots, aName);
		if (slot1 == null) {
			throw new KilimException("attempt to get an unknown slot " + aName + " in a component " + getQualifiedName());
		}
		return slot1;
	}	
	
	/**
	 * returns as an iterator the slots declared in the component.
	 * @return Iterator
	 */
	public Iterator getSlots() {
		if (slots == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return slots.iterator();
	}

	/**
	 * This method is used to register slots in which the component is plugged to. This method is thus invoked internally
	 * by the plug method.
	 * @param aSlot : the slot to be register.
	 * @throws KilimException : generated if aSlot is null or if the component is already registered as a components' plugTo..
	 */
	public void addPlugTo(ComponentSlot aSlot) throws KilimException {
		if (aSlot == null) {
			throw new KilimException("attempt to add a null slot in the list of plugged slotTo of component " + getQualifiedName());
		}
		if (plugTos == null) {
			plugTos = new ArrayList();
		}
		
		if (plugTos.contains(aSlot)) {
			throw new KilimException("attempt to add an already plugged component " + aSlot.getQualifiedName() + " in the list of plugged components of component " + getQualifiedName());	
		}
		plugTos.add(aSlot);
	}

	/**
	 * This method is used to unregister slots in which the component is plugged to. This method is thus invoked internally
	 * by the unplug method.
	 * @param aSlot : the slot to be removed.
	 * @throws KilimException : generated if aSlot is null or if the component is not plugged in aSlot.
	 */
	public void removePlugTo(ComponentSlot aSlot) throws KilimException {
		if (aSlot == null) {
			throw new KilimException("attempt to remove a null  slotTo from component " + getQualifiedName());
		}
		if (plugTos == null) {
			throw new KilimException("attempt to remove a slotTo " + aSlot.getQualifiedName() + " from an unplugged component " + getQualifiedName());
		}
		
		boolean result = plugTos.remove(aSlot);
		if (result == false) {
			throw new KilimException("attempt to remove an unknown slotTo " + aSlot.getQualifiedName() + "  from an component " + getQualifiedName());
		}
	}

	/**
	 * @see org.objectweb.kilim.model.Component#getPlugTos()
	 */
	public Iterator getPlugTos() {
		if (plugTos == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return plugTos.listIterator();
	}
	
	/**
	 * plugs a component in a slot defined in the current component.
	 * @param aName : the local name of the slot.
	 * @param aComponent : the component to be plugged.
	 * @throws KilimException : generated if aName is null, if aName is not the local name of a slot defined in the component or if aComponent is null.
	 */
	public void plug(String aName, Component aComponent) throws KilimException {
		RtComponentSlot slt = (RtComponentSlot) getSlot(aName);
		slt.plug(aComponent);
	}
	
	/**
	 * unplugs a component from a slot defined in the current component.
	 * @param aName : the local name of the slot.
	 * @param aComponent : the component to be plugged.
	 * @throws KilimException : generated if aName is null, if aName is not the local name of a slot defined in the component or if aComponent is null.
	 */
	public void unplug(String aName, Component aComponent) throws KilimException {
		RtComponentSlot slt = (RtComponentSlot) getSlot(aName);
		slt.unplug(aComponent);
		((RtComponent) aComponent).removePlugTo(slt);
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getValue(RuntimeContext)
	 */
	public Object getValue() {
		return this;
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#setEventSource(Object)
	 */
	public void setEventSourceValue(Object aValue) throws KilimException {
		throw new KilimException("attempt to set an event source to a component " + getQualifiedName());
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#getEventSource()
	 */
	public Object getEventSourceValue() throws KilimException {
		throw new KilimException("attempt to get the event source of a component " + getQualifiedName());
	}	
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#isEventSource()
	 */
	public boolean isEventSource() {
		return false;
	}
		
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#addInterfaceListener(MultipleValueInterface)
	 */
	public void addInterfaceListener(RtCollectionPort aInterface) throws KilimException {
		if (nAryListeners == null) {
			nAryListeners = new ArrayList();
		}
		
		nAryListeners.add(aInterface);
	}
	
	/**
	 * @see org.objectweb.kilim.model.RuntimeSource#removeInterfaceListener(MultipleValueInterface)
	 */
	public void removeInterfaceListener(RtCollectionPort aInterface) throws KilimException {
		nAryListeners.remove(aInterface);
	}
	
	/**
	 * returns as an interator the interface  the interface listeners.
	 * @return Iterator
	 */
	public Iterator getInterfaceListeners() {
		if (nAryListeners == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return nAryListeners.listIterator();
	}
	
	/**
	 * creates a copy of the component as defined in the template. The method forks does not account for any operation
	 * performed at runtime (late binding, instanciation, ....).
	 * @return Component
	 * @throws KilimException : generated when invoked on a component having no factory (should not be a KilimException but an InternalException FHO !!!)
	 */
	public Component fork() throws KilimException {
		if (factory == null) {
			throw new KilimException("attempt to fork a component " + getQualifiedName() + " with a null factory ");
		}
		
		RtComponent pCompo = (RtComponent) getContainingComponent();
		if (pCompo != null) {
			Component compo = ((ComponentFactory) pCompo.getFactory()).fork(this);
			return compo;
		} else {
			TemplateDescription tmp = getTemplate();
			Component compo = ComponentFactory.newComponent(tmp);
			if (compo instanceof RtComponent) {	
				((RtComponent) compo).isForkedComponent(true);
			}	
			return compo;
		}
	}
	
	/**
	 * @see org.objectweb.kilim.model.Component#isInitialized()
	 */
	public boolean isInitialized() {
		return isInitialized;
	}
	
	/**
	 * @see org.objectweb.kilim.model.Component#setInitialized(boolean)
	 */
	public void setInitialized() {
		isInitialized = true;
	}
	
	private boolean isForkedComponent() {
		return isForkedComponent;
	}
	
	private void isForkedComponent(boolean iFE) {
		isForkedComponent = iFE;
	}
	
	/**
	 * @see org.objectweb.kilim.model.Component#getInstanciationMgerFromConfiguration()
	 */
	public InstanciationMger getInstanciationMgerFromConfiguration() throws KilimException {
		//lookup for the right instanciation manager.				
		instanciationMger  = null;
		
		//look for a "per instance" manager.
		instanciationMger = KilimConfiguration.getInstanciationStrategy().getPerInstanceMger(getQualifiedName());
		
		//look up of a "per template" manager, if no "per instance" manager.is defined for the current instance.
		if (instanciationMger == null) {
			TemplateDescription template1 = getTemplate();
			TemplateDescription template2 = null;
			do {
				template2 = template1.getSuperTemplate();
				if (template2 != null) {
					template1 = template2;
				}
			} while (template2 != null);

			instanciationMger = KilimConfiguration.getInstanciationStrategy().getPerTemplateMger(template1.getName());
		}
		
		//use of the default manager, if no "per template" manager.is defined for the current instance template.
		if (instanciationMger == null) {
				instanciationMger = KilimConfiguration.getInstanciationStrategy().getDefaultMger();
		}
		
		return instanciationMger;
	}

	/**
	 * @see org.objectweb.kilim.model.Component#getInstanciationMger()
	 */
	public  InstanciationMger getInstanciationMger() {
		return instanciationMger;
	}
	
	/**
	 * @see org.objectweb.kilim.model.Component#setInstanciationMger(InstanciationMger)
	 */
	public  void setInstanciationMger(InstanciationMger aMger) throws KilimException {
		if (aMger == null) {
			throw new KilimException("attempt to install a null instanciation manager to component " + getQualifiedName());
		}
		instanciationMger = aMger;
	}
	
	/**
	 * @see org.objectweb.kilim.model.Component#release()
	 */
	public void release() throws KilimException {
		// remove from parent
      	Component compo = getContainingComponent();
      	boolean isSubCompo = false;
      	if (compo != null) {
     		compo.removeSubComponent(this);
     		isSubCompo = true;
      	}
      	
      ComponentFactory fct = (ComponentFactory) getFactory();
      ComponentFactory nfct =  (ComponentFactory) fct.getContainingFactory();
      if (nfct != null) {
    	nfct.removeSubFactory(fct);
     	nfct.removeChildNamingContext(fct.getLocalName());
     	fct.setParentNamingContext(null);

     	if (isSubCompo) {
     		nfct.removeBoundName(getLocalName());
     	}
      }

		// unplug explicitly...
 		ArrayList list = null;
 		for (Iterator iter = getPlugTos(); iter.hasNext();) {
      		if (list == null) {
      			list = new ArrayList();
      		}
			list.add(iter.next());
		}
		      
      	if (list != null) {
      		for (Iterator iter = list.iterator(); iter.hasNext();) {
      			RtComponentSlot slt = (RtComponentSlot) iter.next();
	       		slt.unplug(this);
      		}
      	}
	}
	
	/**
	 * This method is an extension introduced to ease the mapping with the FRACTAL model : a controller is a technical object internally used by the component
	 * (most controllers are used as instantiation managers or communication managers) according to its specific semantics. 
	 * Controllers are thus just stored in a table contained in the kilim component object. This method registers a new controller in the component.
	 * @param aName : the local name of the controller.
	 * @param aController : the controller.
	 * @throws KilimException : generated if aName or aController is null.
	 */
	public void addController(String aName, Object aController) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to add a controller with a null name in component " + getQualifiedName());
		}		
		if (aController == null) {
			throw new KilimException("attempt to add a null controller in component " + getQualifiedName());
		}
		if (controllers == null) {
			controllers = new HashMap();
		}
			
		controllers.put(aName, aController);
	}
	
	/**
	 * This method is an extension introduced to ease the mapping with the FRACTAL model : a controller is a technical object internally used by the component
	 * (most controllers are used as instantiation managers or communication managers) according to its specific semantics. 
	 * Controllers are thus just stored in a table contained in the kilim component object. This method removes a previously registered controller.
	 * @param aName : the local name of the controller.
	 * @throws KilimException : generated if aName is null or if the controller is not defined in the component.
	 */
	public void removeController(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to remove a null controller in component " + getQualifiedName());
		}
		if (controllers == null) {
			throw new KilimException("attempt to remove a controller from component " + getQualifiedName() + " without controller");
		}
		
		Object result = controllers.remove(aName);
		if (result == null) {
			throw new KilimException("attempt to remove an unknown controller " + aName + "  from component " + getQualifiedName());
		}
	}
	
	/**
	 * This method is an extension introduced to ease the mapping with the FRACTAL model : a controller is a technical object internally used by the component
	 * (most controllers are used as instantiation managers or communication managers) according to its specific semantics. 
	 * Controllers are thus just stored in a table contained in the kilim component object. This method returns the controller identified by its name.
	 * @param aName : the name of the controller
	 * @return Object
	 * @throws KilimException : generated if aName is null or if the controller is not defined in the component.
	 */
	public Object getController(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get a controller through a null name in component " + getQualifiedName());
		}
		
		if (controllers == null) {			
			throw new KilimException("attempt to get a controller " + aName + " from a component " + getQualifiedName() + " without controller");
		}

		Object ctrl = controllers.get(aName);
		if (ctrl == null) {
			throw new KilimException("attempt to get an unknown controller " + aName + " in a component " + getQualifiedName());
		}
		return ctrl;
	}
	
	/**
	 *	This method is an extension introduced to ease the mapping with the FRACTAL model : a controller is a technical object internally used by the component
	 * (most controllers are used as instantiation managers or communication managers) according to its specific semantics. 
	 * Controllers are thus just stored in a table contained in the kilim component object. This method returns as an iterator the controllers defined in the component.
	 * @return Iterator
	 */
	public Iterator getControllers() {
		if (controllers == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return controllers.values().iterator();
	}
}