/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NamedElement;
import org.objectweb.kilim.description.Port;
import org.objectweb.kilim.description.Slot;
import org.objectweb.kilim.description.TemplateElementImpl;
import org.objectweb.kilim.description.Trigger;

/**
 * @author horn
 */
	
public class RtComponentSlot extends RtComponentElement implements ComponentSlot {
	private SlotFactory factory;
	private boolean isInitialized;
	private List interfaces;
	private List plugIns;
	
	/**
	 * The public constructor for RtComponentSlots
	 * @param aSlot : the slot template description
	 * @param aComponent : the component containing the slot
	 * @param aFactory : the slot factory.
	 */
	public RtComponentSlot(Slot aSlot, Component aComponent, SlotFactory aFactory) {
		super(aSlot, aComponent);
		factory = aFactory;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ContainerElement#isComponent()
	 */
	public boolean isComponent() {
		return false;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ContainerElement#isSlot()
	 */
	public boolean isSlot() {
		return true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ContainerElement#getFactory()
	 */
	public Factory getFactory() {
		return factory;
	}
	
	/**
	 * @see org.objectweb.kilim.model.Component#isInitialized()
	 */
	public boolean isInitialized() {
		return isInitialized;
	}
	
	/**
	 * @see org.objectweb.kilim.model.Component#setInitialized(boolean)
	 */
	public void setInitialized() {
		isInitialized = true;
	}
	
	/**
	 * @see org.objectweb.kilim.model.ComponentElement#getLocalName()
	 */
	public String getLocalName() {
		NamedElement elem = (NamedElement) getElementDescription();
		return elem.getLocalName();
	}

	/**
	 * adds an Interface.in the slot.
	 * @param aInterface : the interface to be added.
	 * @throws KilimException : generated if aInterface is null or is already present in the slot.
	 */
	public void addInterface(ComponentInterface aInterface) throws KilimException {
		if (aInterface == null) {
			throw new KilimException("attempt to add a null interface in slot " + getQualifiedName());
		}
		
		if (!(aInterface instanceof RtComponentInterface)) {
			throw new KilimException("attempt to add a non kilim interface in slot " + getQualifiedName());
		}
		
		if (interfaces == null) {
			interfaces = new ArrayList();
		}
		String lName = aInterface.getLocalName();
		if (containsElement(interfaces, lName)) {
			throw new KilimException("attempt to add an already added interface " + aInterface.getQualifiedName() + " in slot " + getQualifiedName());
		}
		interfaces.add(aInterface);
	}
	
	/**
	 * removes an interface. from the slot.
	 * @param aInterface : the interface to be removed.
	 * @throws KilimException : generated if aInterface is null or is unknown.
	 */
	public void removeInterface(ComponentInterface aInterface) throws KilimException {
		if (aInterface == null) {
			throw new KilimException("attempt to remove a null interface from slot " + getQualifiedName());
		}
		if (interfaces == null) {
			throw new KilimException("attempt to remove an interface" + aInterface.getLocalName() + " from slot " + getQualifiedName());
		}
		String lName = aInterface.getLocalName();
		Object previous = removeElement(interfaces, lName);
		if (previous == null) {
			throw new KilimException("attempt to remove an unknown interface" + lName + " from slot " + getQualifiedName());
		}
	}

	/**
	 * returns as an interator the interfaces known inthe slot.
	 * @return Iterator
	 */
	public Iterator getInterfaces() {
		if (interfaces == null) {
			return KILIM.EMPTY_ITERATOR;
		}
		return interfaces.iterator();
	}
	
	/**
	 * returns the local interface identified by its local name.
	 * @param aName : the local name of the interface.
	 * @return ComponentInterface :
	 * @throws KilimException : generated if aName is null or if the slot does not contain an interface whose name is aName.
	 */
	public ComponentInterface getInterface(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get an interface through a null name in slot " + getQualifiedName());
		}
		if (interfaces == null) {
			throw new KilimException("attempt to get interface " + aName + " from empty slot " + getQualifiedName());
		}
		
		ComponentInterface result = (ComponentInterface)  getElement(interfaces, aName);
		if (result == null) {
			throw new KilimException("attempt to get a unknown interface " + aName + " from slot " + getQualifiedName());
		}
		return result;
	}
	
	/**
	 * returns as an iterator the components plugged in the slot..
	 * @return Iterator
	 */
	public Iterator getPlugIns() {
		if (plugIns == null) {
			return KILIM.EMPTY_ITERATOR;
		}		
		return plugIns.listIterator();
	}
	
	/**
	 * plugs a component in the slot.
	 * @param aComponent : the component to be plugged.
	 * @throws KilimException : generated if aComponent is null or is already plugged in the slot.
	 */
	public void plug(Component aComponent) throws KilimException {
		if (aComponent == null) {
			throw new KilimException("attempt to plug a null Component in slot " + getQualifiedName());
		}
		
		if (plugIns == null) {
			plugIns = new ArrayList();
		}
		
		if (interfaces == null) {
			return;
		}
		
		if (plugIns.contains(aComponent)) {
			throw new KilimException("attempt to plug a Component " + aComponent.getQualifiedName() + " already present in slot " + getQualifiedName());
		}
		
		Iterator iter = interfaces.iterator();
		
		while (iter.hasNext()) {
			RtComponentInterface interf = (RtComponentInterface)  iter.next();
			String lName = interf.getLocalName();
			if (interf == null) {
				throw  new KilimException ("attempt to use an unknown offered port " + lName + " when plugging a component " + aComponent + " into slot " + getQualifiedName());
			}
			
			RtComponentInterface interf1 = (RtComponentInterface) aComponent.getInterface(lName);
			
			// The following code replaces the previous commented lines : only roles of ports defined in slots are considered.
			//The role of the component ports is considered as meaningless by Kilim2.
			boolean isOff = true;
			TemplateElementImpl elDesc = interf.getElementDescription();
			boolean isOff1 = true;
			TemplateElementImpl elDesc1 = interf1.getElementDescription();
			
			if (elDesc instanceof Port) {
				isOff = ((Port) elDesc).isOffered();
				if (isOff) {
					if (elDesc1 instanceof Port && !((Port) elDesc1).isOffered()) {
						//System.err.println("warning : port " + interf1.getQualifiedName() + " is declared <required> in component " + aComponent.getQualifiedName() + " and <offered> in slot " + getQualifiedName());
					}
				} else {
					if (!(elDesc1 instanceof Port) || ((Port) elDesc1).isOffered()) {
						//System.err.println("warning : interface " + interf1.getQualifiedName() + " is declared <offered> in component " + aComponent.getQualifiedName() + " and <required> in slot " + getQualifiedName());
					}
				}
			} else {
				if (!(elDesc1 instanceof Port) || !((Port) elDesc1).isOffered()) {
					//System.err.println("warning : interface " + interf1.getQualifiedName() + " is declared <required> in component " + aComponent.getQualifiedName() + " and <offered> in slot " + getQualifiedName());
				}
			}
			
			if (elDesc1 instanceof Port) {
				isOff = ((Port) elDesc).isOffered();
				isOff1 = ((Port) elDesc1).isOffered();
				if (isOff && (elDesc1 instanceof Port && !isOff1)) {
					//System.out.println("warning : port " + interf1.getQualifiedName() + " is declared <required> in component " + aComponent.getQualifiedName() + " and <offered> in slot " + getQualifiedName());
				}
			}
			
			if (isOff) {
				interf.bindProvider(interf1, false);
				if (isInitialized() || aComponent.isInitialized()) {
					if (interf instanceof RtCollectionPort || interf1.hasValue()) {
						interf.fireTriggers(Trigger.BIND, interf1.getValue());
					}
				}
			} else {
				interf1.bindProvider(interf, false);
				if (isInitialized() || aComponent.isInitialized()) {
					//list.add(new InterfacePair(interf1, interf));
					if (interf1 instanceof RtCollectionPort  || interf.hasValue()) {
						interf1.fireTriggers(Trigger.BIND, interf.getValue());
					}
				}
			}
		}
		
		plugIns.add(aComponent);
		((RtComponent)  aComponent).addPlugTo(this);
	}
	
	/**
	 * unplugs a component from the slot.
	 * @param aComponent : the component to be removed.
	 * @throws KilimException : generated if aComponent is null or is not plugged in the slot.
	 */
	public void unplug(Component aComponent) throws KilimException {
		if (aComponent == null) {
			throw new KilimException("attempt to unplug a null component in slot " + getQualifiedName());
		}
		
		if (plugIns == null) {
			throw new KilimException("attempt to unplug a component \"" + aComponent.getQualifiedName() + "\" from an unplugged slot \"" + getQualifiedName() + "\"");
		}
		
		boolean remove = plugIns.remove(aComponent);
		if (!remove) {
			throw new KilimException("attempt to unplug a non plugged component \"" + aComponent.getQualifiedName() + "\" from slot \"" + getQualifiedName() + "\"");
		}
		
		((RtComponent) aComponent).removePlugTo(this);

		Iterator iter = getInterfaces();
		List list = new ArrayList();
		
		while (iter.hasNext()) {
			RtComponentInterface interf = (RtComponentInterface)  iter.next();
			String lName = interf.getLocalName();
			
			if (interf == null) {
				throw  new KilimException ("attempt to use an unknown offered port " + lName + " when unplugging a component " + aComponent.getQualifiedName() + " from slot " + getQualifiedName());
			}
			
			RtComponentInterface interf1 = (RtComponentInterface) aComponent.getInterface(lName);
			TemplateElementImpl elDesc = interf.getElementDescription();
			boolean isOff = true;
			
			if (elDesc instanceof Port) {
				isOff = ((Port) elDesc).isOffered();
			}
						
			if (isOff) {
				interf.unbindProvider(interf1);
				//list.addLast(new InterfacePair(interf, interf1));
				if (interf1.hasValue()) {
					interf.fireTriggers(Trigger.UNBIND, interf1.specificGetValue());
				}
			} else {
				interf1.unbindProvider(interf);
				//list.addLast(new InterfacePair(interf1, interf));
				if (interf.hasValue()) {
					interf1.fireTriggers(Trigger.UNBIND, interf.specificGetValue());
				}
			}
		}
	}
}

class InterfacePair {
	RtComponentInterface first;
	RtComponentInterface second;	
	
	InterfacePair(RtComponentInterface itf1, RtComponentInterface itf2) {
		first = itf1;
		second = itf2;
	}
}