/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.instanciation;

import java.util.Iterator;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.Trigger;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentInterface;
import org.objectweb.kilim.model.ComponentSlot;
import org.objectweb.kilim.model.RtCollectionPort;
import org.objectweb.kilim.model.RtComponentInterface;
import org.objectweb.kilim.model.RtComponentSlot;
import org.objectweb.kilim.model.RuntimeAction;
import org.objectweb.kilim.model.RuntimeSource;
import org.objectweb.kilim.model.RuntimeTrigger;
import org.objectweb.kilim.model.mapping.MappingContext;

/**
 * @author horn
 */
	
public class BDUInstanciationMger extends InstanciationMger {
	
	/**
	 * @see org.objectweb.kilim.model.InstanciationMger#finalizeInstanciation(TemplateDescription)
	 */
	public void finalizeInstanciation(Component aComponent, ComponentInterface aInterface, MappingContext aContext) throws KilimException { }
	
	/**
	 * @see org.objectweb.kilim.model.InstanciationMger#initializeInstanciation(Component)
	 */
	public void initializeInstanciation(Component aComponent, ComponentInterface aInterface, MappingContext aContext) throws KilimException {
		doWhenInstanciate(aComponent, aInterface, aContext);					
	}
	
	/**
	 * @see org.objectweb.kilim.model.services.InstanciationMger#initializePlug(Component, ComponentSlot, MappingContext)
	 */
	public void initializePlug(Component aComponent, ComponentSlot aSlot, MappingContext aContext) throws KilimException {
		doWhenPlug(aComponent, aSlot, aContext);
	}	
	
	/**
	 * @see org.objectweb.kilim.model.services.InstanciationMger#finalizePlug(Component, ComponentSlot, MappingContext)
	 */
	public void finalizePlug(Component aComponent, ComponentSlot aSlot, MappingContext aContext) throws KilimException { }
		
	/**
	 * Method doWhenInstanciate performs the component instanciation according to the following rules : it first looks up for all the nary ports
	 * directly declared at the component level and invokes the getValue() method on every provider they contain. It then looks up for 
	 * all the nary ports declared in slots and invokes the getValue() method on every provider they contain. No getValue() on single 
	 * value port is performed without need. 
	 * @param aComponent is the component containing the current interfarface.
	 * @param aInterface is the current interface.
	 * @param aContext :
	 * @throws KilimException is generated when aComponent is null.
	 */

	public static void doWhenInstanciate(Component aComponent, ComponentInterface aInterface, MappingContext aContext) throws KilimException {
		if (aComponent == null) {
			throw new KilimException("attempt to invoke an instanciation manager on a null component");
		}
		
		if (aComponent.isInitialized()) {
			return;
		}				
		aComponent.setInitialized();
		
		Component contain = aComponent.getContainingComponent();
		while (contain != null && !contain.isInitialized()) {
			doWhenInstanciate(contain, null, aContext);
		}

		Iterator iter0 = aComponent.getInterfaces();
		while (iter0.hasNext()) {
			RtComponentInterface interf = (RtComponentInterface) iter0.next();
			if (interf instanceof RtCollectionPort) {
				doWhenInstanciate1((RtCollectionPort) interf, aContext);
	 		}
		}
		
		Iterator iter1 = aComponent.getSlots();
		while (iter1.hasNext()) {
			RtComponentSlot  slot  = (RtComponentSlot) iter1.next();
			doWhenPlug(aComponent, slot, aContext);
		}
	}
	
	private static void doWhenInstanciate1(RtCollectionPort aInterface, MappingContext aContext) throws KilimException {
		Iterator iter = aInterface.getBoundProviders();
		while (iter.hasNext()) {
			ComponentInterface provider = (ComponentInterface) iter.next();
			if (provider instanceof RtCollectionPort) {
				//doWhenInstanciate1((RtCollectionPort) provider, aContext);
				throw new KilimException("attempt to use a nary port " + provider.getQualifiedName() + " inside another nary port " + aInterface.getQualifiedName());
			} else {
				RuntimeSource rtSource = (RuntimeSource) ((RuntimeSource) provider).getTarget();
				if (!aInterface.triggersDone(rtSource, Trigger.BIND)) {
					aInterface.setTriggersDone(rtSource, Trigger.BIND, true);					
					Iterator iter2 = aInterface.getTriggers(Trigger.BIND);
					Object resultValue = provider.getValue();
	 				while (iter2.hasNext()) {
						RuntimeTrigger rtElem = (RuntimeTrigger) iter2.next();
						if (rtElem.getEventKind() == Trigger.BIND) {
							Iterator iter3 = rtElem.getTransformers();
							while (iter3.hasNext()) {
								RuntimeAction rtElem3 = (RuntimeAction) iter3.next();
								rtElem3.setEventSourceValue(resultValue);
								
								rtElem3.execute();
							}
						}
					}
				}
			}
	 	}
	}
	
	/**
	 * Method doWhenPlug.
	 * @param aComponent :
	 * @param aSlot :
	 * @param aContext :
	 * @throws KilimException :
	 */
	public static void doWhenPlug(Component aComponent, ComponentSlot aSlot, MappingContext aContext) throws KilimException {
		if (aSlot.isInitialized()) {
			return;
		}				
		aSlot.setInitialized();

		Iterator iter0 = aSlot.getInterfaces();
		while (iter0.hasNext()) {
			RtComponentInterface interf = (RtComponentInterface) iter0.next();
			if (interf instanceof RtCollectionPort) {
				doWhenInstanciate1((RtCollectionPort) interf, aContext);
	 		}	
		}
	}
}