/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.instanciation;

import org.objectweb.kilim.KilimException;

/**
 * This class is a simple container dedicated to the definition of instanciation strategies. It is used by the component
 * factory to find the instanciation manager to be associated to each application component. It is possible to redefine
 * a default instanciation manager, and to define per template managers and per instance managers. 
 * The factory uses the following look up strategy. The search is stopped as soon as a manager is found.
 * the component factory first looks for a manager explicitely associated to the component.
 * If none can be found, it then looks for a manager explicitely associated to the component  template.
 * If none can be found, it recursively looks for a manager explicitely associated to super templates.
 * If none can be found it looks for a default manager.
 * 
 * No null value can be returned, since this class defines a NAryPreInstanciationMger as default instanciation manager
 * and no null value is accepted by the setDefaultMger().
 * @author horn
 */

public interface InstanciationStrategy {
	
	/**
	 * Method setDefaultMger.
	 * @param aMger : a default instanciation manager (this ctor just contains a call to setDefaultManager(aMger). This value
	 * cannot be null. A NullInstanciationMger should be indicated if no action should be performed at instanciation time.
	 * @throws KilimException : generated if the reference is null. 
	 */
	void setDefaultMger(InstanciationMger aMger) throws KilimException;
	
	/**
	 * Method getDefaultMger returns the default manager.
	 * @return InstanciationMger
	 */
	InstanciationMger getDefaultMger();

	/**
	 * Method setPerTemplateMger sets the manager to be used when the component is an instance of a template or one
	 * of its subtemplates.
	 * @param aName is name of the template.
	 * @param aManager is the manager to be used.
	 * @throws KilimException : generated when aName is null.
	 */
	void setPerTemplateMger(String aName, InstanciationMger aManager) throws KilimException;
	
	/**
	 * Method getPerTemplateMger 
	 * @param aName : the name of the template.
	 * @return InstanciationMger
	 * @throws KilimException : generated when aName is null
	 */
	InstanciationMger getPerTemplateMger(String aName) throws KilimException;
	
	/**
	 * Method setPerInstanceMger sets the manager to be used for a given instance.
	 * @param aName : the name of the component.
	 * @param aManager : the manager to be used
	 * @throws KilimException : generated when aName is null.
	 */
	void setPerInstanceMger(String aName, InstanciationMger aManager) throws KilimException;
	
	/**
	 * Method getPerInstanceMger.
	 * @param aName : the name of the component
	 * @return InstanciationMger :
	 * @throws KilimException : generated when aName is null.
	 */
	InstanciationMger getPerInstanceMger(String aName) throws KilimException;
}
