/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.mapping;

import java.util.HashMap;

import org.objectweb.kilim.KilimException;

/**
 * This class is a simple container dedicated to the definition of instanciation strategies. It is used by the component
 * factory to find the instanciation manager to be associated to each application component. It is possible to redefine
 * a default instanciation manager, and to define per template managers and per instance managers. 
 * The factory uses the following look up strategy. The search is stopped as soon as a manager is found.
 * the component factory first looks for a manager explicitely associated to the component.
 * If none can be found, it then looks for a manager explicitely associated to the component  template.
 * If none can be found, it recursively looks for a manager explicitely associated to super templates.
 * If none can be found it looks for a default manager.
 * 
 * No null value can be returned, since this class defines a NAryPreInstanciationMger as default instanciation manager
 * and no null value is accepted by the setDefaultMger().
 * @author horn
 */

public class DefaultMappingStrategy implements MappingStrategy {
	private Mapper  defaultMapper = new JavaRuntimeMapper();
	private HashMap perTemplateMapper;
	private HashMap perInstanceMapper;
	
	/**
	 * @see java.lang.Object#Object()
	 */
	public DefaultMappingStrategy() throws KilimException {
	}
	
	/**
	 * Method DefaultMappingStrategy.
	 * @param aMapper : a default  mapper (this ctor just contains a call to setDefaultMapper(aMapper)
	 * @throws KilimException : generated if the reference is null. A NullMapper should be indicated if
	 *  no mapping should be performed at runtime.
	 */
	public DefaultMappingStrategy(Mapper aMapper) throws KilimException {
		setDefaultMapper(aMapper);
	}

	/**
	 * @see org.objectweb.kilim.model.mapping.MappingStrategy#setDefaultMapper(Mapper)
	 */
	public void setDefaultMapper(Mapper aMapper) throws KilimException {
		if (aMapper == null) {
			throw new KilimException("attempt to install a null  mapper as default");
		}
		defaultMapper = aMapper;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.MappingStrategy#getDefaultMapper()
	 */
	public Mapper getDefaultMapper() {
		return defaultMapper;
	}

	/**
	 * @see org.objectweb.kilim.model.mapping.MappingStrategy#setPerTemplateMapper(String, Mapper)
	 */
	public void setPerTemplateMapper(String aName, Mapper aMapper) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to install a manager for template through a null name");
		}

		if (perTemplateMapper == null) {
			perTemplateMapper = new HashMap();
		}
		perTemplateMapper.put(aName, aMapper);
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.MappingStrategy#getPerTemplateMapper(String)
	 */
	public Mapper getPerTemplateMapper(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get a template manager through a null name ");
		}
		
		if (perTemplateMapper == null) {
			return null;
		}
		return (Mapper) perTemplateMapper.get(aName);
	}

	/**
	 * @see org.objectweb.kilim.model.mapping.MappingStrategy#setPerInstanceMapper(String, Mapper)
	 */
	public void setPerInstanceMapper(String aName, Mapper aMapper) throws KilimException {
		if (aMapper == null) {
			throw new KilimException("attempt to install a null mapper for template " + aName);
		}

		if (perInstanceMapper == null) {
			perInstanceMapper = new HashMap();
		}
		perInstanceMapper.put(aName, aMapper);
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.MappingStrategy#getPerInstanceMapper(String)
	 */
	public Mapper getPerInstanceMapper(String aName) throws KilimException {
		if (aName == null) {
			throw new KilimException("attempt to get a instance manager through a null name ");
		}
		
		if (perInstanceMapper == null) {
			return null;
		}
		return (Mapper) perInstanceMapper.get(aName);
	}
}
