/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.mapping;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.model.services.DefaultRuntimeClassLoader;

/**
 * This mapper 
 * @author horn
 */
public class JavaRuntimeMapper implements Mapper {
	PrintWriter pW=null;
	FileWriter fW = null;
	
	/**
	 * @see org.objectweb.kilim.model.mapping.Mapper#enterContext(Object). This method is empty.
	 */
	public void enterContext(MappingContext aContext) throws KilimException { }
	
	/**
	 * @see org.objectweb.kilim.model.mapping.Mapper#leaveContext(). This method is empty.
	 */
	public void leaveContext(MappingContext aContext) throws KilimException { }

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getGetterValue(Object, boolean, String, MappingContext). This method returns
	 * the value of the field "filedName" of the class (if isStatic is true) or object (if isStatic is false) referenced by aSupport. The KilimException
	 * is here a wrapper to all exceptions generated by the java reflexive API when trying to perform the task.
	 */
	public Object getGetterValue (Object aSupport, boolean isStatic, String fieldName, MappingContext aContext) throws KilimException {		   
		
		Class clazz = null;	
		if (aSupport instanceof Class && isStatic) {
			clazz = (Class) aSupport;
		}	else {
			clazz = aSupport.getClass();
		}	
		
		Field field = null;		
		try {
			field = clazz.getField(fieldName);
		} catch (Exception exc) {
			throw  new KilimException(exc);
		}
		
		if (field == null) {
			throw  new KilimException("attempt to get the value of an unknown field " + fieldName + " in setter of " + clazz.getName());
		}
		
		boolean jStatic = (field.getModifiers() & Modifier.STATIC) != 0;
		if (isStatic && !jStatic) {
			throw   new KilimException("attempt to acces as static a non static field " + fieldName + " in " + clazz.getName());
		}
		
		if (!isStatic && jStatic) {
			throw   new KilimException("attempt to acces as non static a static field " + fieldName + " in " + clazz.getName());
		}
		
		Object resultValue = null;
		try {
			resultValue = field.get(aSupport);
		} catch (Exception ex) {
			throw  new KilimException(ex);
		}
			
		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeSetter(Object, boolean, String, Object, MappingContext). This method sets
	 * a value in the field "filedName" of the class (if isStatic is true) or object (if isStatic is false) referenced by aSupport. The KilimException
	 * is here a wrapper to all exceptions generated by the java reflexive API when trying to perform the task.
	 */
	public void executeSetter(Object aSupport, boolean isStatic, String fieldName, Object toBeSet, MappingContext aContext) throws KilimException {
		Class clazz = null;
		if (aSupport instanceof Class && isStatic) {
			clazz = (Class) aSupport;
		} else {
			clazz = aSupport.getClass();
		}

		Field field = null;			
		try {
			field = clazz.getField(fieldName);
		} catch (Exception exc) {
			throw  new KilimException(exc);
		}
		
		if (field == null) {
			throw  new KilimException("attempt to access an unknown field " + fieldName + " in setter of " + clazz.getName());
		}
		
		boolean jStatic = (field.getModifiers() & Modifier.STATIC) != 0;
		if (isStatic && !jStatic) {
			throw   new KilimException("attempt to acces as static a non static field " + fieldName + " in " + clazz.getName());
		}
		
		if (!isStatic && jStatic) {
			throw   new KilimException("attempt to acces as non static a static field " + fieldName + " in " + clazz.getName());
		}			
			
			if (toBeSet == null) {
				throw  new KilimException("attempt to set a non initialized  value to field " + fieldName + " in " + clazz.getName());
			}			
			
		try {
			field.set(aSupport, toBeSet);
		} catch (Exception exc) {
			throw  new KilimException(exc);
		}
	}

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getMethodValue(Object, boolean, String, Object[], String[], MappingContext). This method returns
	 * the value returned by the execution of the method  "methodName" of the class (if isStatic is true) or object (if isStatic is false) referenced by aSupport. 
	 * The KilimException is here a wrapper to all exceptions generated by the java reflexive API when trying to perform the task.
	 */
	public Object getMethodValue(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {
	
		if (aSupport == null) {
			throw new KilimException("attempt to invoke method \"" + aMethodName + "()\" on a null target.");
		}

		Class clazz = isStatic ? (Class) aSupport : aSupport.getClass();

		int paramNumber = paramObjects.length;
		Class[] paramTypes = new Class[paramNumber];
		for (int i = 0; i < paramNumber; i++) {
			paramTypes[i] = DefaultRuntimeClassLoader.instance.getClass(typeNames[i]);
		}

		Method method = null;
		try {
			method = clazz.getMethod(aMethodName, paramTypes);
		} catch (Exception exc) {
			throw new KilimException(exc);
		}

		if (method == null) {
			throw new KilimException("attempt to acces an unknown or ill declared method " + aMethodName + " in class " + clazz.getName());
		}

		boolean jStatic = (method.getModifiers() & Modifier.STATIC) != 0;
		if (isStatic && !jStatic) {
			throw new KilimException("attempt to acces a non static method as static  " + aMethodName + " in class " + clazz.getName());
		}

		if (!isStatic && jStatic) {
			throw new KilimException("attempt to acces a static method as non static  " + aMethodName + " in class " + clazz.getName());
		}

		Object resultValue = null;
		try {
			resultValue = method.invoke(aSupport, paramObjects);
		} catch (Exception e) {
			e.printStackTrace();
			StringBuffer buff = new StringBuffer("exception while calling :\n\"" + aSupport + "\"[" + aSupport.getClass() + "]." + aMethodName + "(");
			for (int i = 0; i < paramObjects.length; i++) {
				buff.append("\"");
				if (paramObjects[i] != null) {
					buff.append(paramObjects[i].toString());
					buff.append("\"[");
					buff.append(paramObjects[i].getClass());
					buff.append("]");
				} else {
					buff.append("\"null\" [unknown]");
				}
				if (i != 0) {
					buff.append(",");
				}
			}
			buff.append(").");
			throw new KilimException(buff.toString());
		}

		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeMethod(Object, boolean, String, Object[], String[], MappingContext)
	 */
	public void executeMethod(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {
		getMethodValue(aSupport, isStatic, aMethodName, paramObjects, typeNames, aContext);
	}	
			
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getConstructorValue(Class, Object[], String[], MappingContext)
	 */
	public Object getConstructorValue(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {		
		
		int paramNumber = paramObjects.length;
		Class[] paramTypes = new Class[paramNumber];

		for (int i = 0; i < paramNumber; i++) {
			paramTypes[i] =	DefaultRuntimeClassLoader.instance.getClass(typeNames[i]);
		}

		Constructor method = null;
		try {
			method = aClass.getConstructor(paramTypes);
		} catch (Exception exc) {
			throw new KilimException(exc);
		}

		if (method == null) {
			throw new KilimException("attempt to use an unknown constructor in class " + aClass);
		}

		Object resultValue = null;
		try {
			resultValue = method.newInstance(paramObjects);
		} catch (Exception e) {
			e.printStackTrace();
			StringBuffer buff = new StringBuffer("exception while calling constructor " + aClass.getName() + "(");
			for (int i = 0; i < paramObjects.length; i++) {
				if (paramObjects[i] != null) {
					buff.append(paramObjects[i].toString());
					buff.append("\"[");
					buff.append(paramObjects[i].getClass());
					buff.append("]");
				} else {
					buff.append("\"null\" [unknown]");
				}
				buff.append(").");
				throw new KilimException(buff.toString());
			}
		}

		return resultValue;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeConstructor(Class, Object[], String[], MappingContext)
	 */
	public void executeConstructor(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException {
		getConstructorValue(aClass, paramObjects, typeNames, aContext);
	}			
			
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getExternalValue(Object, MappingContext)
	 */
	public Object getExternalValue(Object value, MappingContext aContext) {
		return value;
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getPropertyValue(Object, MappingContext)
	 */
	public Object getPropertyValue(Object aValue, MappingContext aContext) {
			return aValue;
	}
		
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getClassValue(String, MappingContext)
	 */
	public Object getClassValue(String aClassName, MappingContext aContext) throws KilimException {
		return DefaultRuntimeClassLoader.instance.getClass(aClassName);
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getEventSourceValue(MappingContext)
	 */
	public Object getEventSourceValue(MappingContext aContext) throws KilimException {
		throw  new KilimException("attempt to get a value directly from an event source");
	}
	
	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#getNullElementValue(MappingContext)
	 */
	public Object getNullElementValue(MappingContext aContext) {
		return null;
	}

	/**
	 * @see org.objectweb.kilim.model.mapping.KilimMapper#executeNullElement(MappingContext)
	 */
	public void executeNullElement(MappingContext aContext)  { }
}