/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.mapping;

import org.objectweb.kilim.KilimException;

/**
 * The Kilim architecture enforces a clear separation between the mechanims implementing the generic Kilim model semantics (mechanisms controlling
 * the execution of providers and transformers in accordance with dependancy rules, trigger rules, etc....) and the mechanisms implementing the effective
 * tasks described by the providers and the transformers (mechanisms to execute constructors, methods, ....). This architecture allows thus different 
 * implementations for the action of providers and transformers : effective instantiation of the system through method and constructor calls,simulation (without
 * effective instantiation) of the system, code generation of launchers, .... A Mapper implements the effective task associated to providers (through getValue()).
 * Mappers should be written in order to be used as interceptors and to be combined in pipe fashion.
 * and transformers (through execute()).
 * @author horn
 */
public interface Mapper {
	
	/**
	 * Method enterContext. Mappers have access to a shared context that stores a global view of the system. the Kilim infrastructure updates this context according
	 * to a stack policy (it is updated every time it invokes the method getValue() or execute()). Method enterContext is a callBack provided by mappers invoked by
	 * the Kilim infrastructure when invoking the method getValue() or execute(). This callback exists because the execution of the effective task associated to these 
	 * methods does not necessarily folow immediately the invocation: they can only be applied on "instanciated" components.and ew getValue() calls and execute() 
	 * may be recursively introduced by instanciation managers.
	 * @param aContext : the mapping context
	 * @throws KilimException : generated by a specific mapper to indicate a (specific problem).
	 */
	void enterContext(MappingContext aContext) throws KilimException;
	
	/**
	 * Method leaveContext. This method is invoked by the Kilim infrastructure when leaving the method getValue() or execute(). This method is invoked just after the
	 * execution of the effective task associated to the provider or transformer.
	 * @param aContext : the mapping context.
	 * @throws KilimException : generated by a specific mapper to indicate a (specific problem).
	 */
	void leaveContext(MappingContext aContext) throws KilimException;

	/**
	 * Method getGetterValue. This method performs the effective task associated to the method getValue() of a getter.
	 * @param aSupport : the support of the getter
	 * @param isStatic : true if the getter is static
	 * @param fieldName : the name of the field to be accessed
	 * @param aContext : the mapping context
	 * @return Object : the value returned by getValue()
	 * @throws KilimException : generated if aSupport is null, if fieldName is null or if no getter corresponding to the signature exists.
	 * Also generated by a specific mapper to indicate a (specific problem).
	 */
	Object getGetterValue (Object aSupport, boolean isStatic, String fieldName, MappingContext aContext) throws KilimException;
	
	/**
	 * Method executeSetter. This method performs the effective task associated to the method execute() of a getter.
	 * @param aSupport : the support of the getter
	 * @param isStatic : true if the getter is static
	 * @param fieldName : the name of the field to be accessed
	 * @param toBeSet : the value to be set.
	 * @param aContext : the mapping context
	 * @throws KilimException : generated if aSupport is null, if fieldName is null or if no getter corresponding to the signature exists.
	 * Also generated by a specific mapper to indicate a (specific problem).

	 */
	void executeSetter(Object aSupport, boolean isStatic, String fieldName, Object toBeSet, MappingContext aContext) throws KilimException;

	/**
	 * Method getMethodValue. This method performs the effective task associated to the method getValue() of a factory method.
	 * @param aSupport : the support of the method
	 * @param isStatic : true if the method is static
	 * @param aMethodName : the name of the method
	 * @param paramObjects : the ordered values of the method parameters
	 * @param typeNames : the ordered types of the method parameters.
	 * @param aContext : the mapping context.
	 * @return Object : the value returned by getValue()
	 * @throws KilimException :generated if aSupport is null, if methodName is null or if no method corresponding to the signature exists.
	 * Also generated by a specific mapper to indicate a (specific problem).
	 */
	Object getMethodValue(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException;
	
	/**
	 * Method executeMethod.  This method performs the effective task associated to the method execute() of a factory method (used as a transformer).
	 * @param aSupport : the support of the method
	 * @param isStatic : true if the method is static
	 * @param aMethodName : the name of the method
	 * @param paramObjects : the ordered values of the method parameters
	 * @param typeNames : the ordered types of the method parameters.
	 * @param aContext : the mapping context.
	 * @throws KilimException :generated if aSupport is null, if methodName is null or if no method corresponding to the signature exists.
	 * Also generated by a specific mapper to indicate a (specific problem).
	 */
	void executeMethod(Object aSupport, boolean isStatic, String aMethodName, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException;
			
	/**
	 * Method getConstructorValue. This method performs the effective task associated to the method getValue() of a constructor.
	 * @param aClass : the class for the constructor
	 * @param paramObjects : the ordered values of the method parameters
	 * @param typeNames : the ordered types of the method parameters.
	 * @param aContext : the mapping context.
	 * @return Object : the value returned by getValue().
	 * @throws KilimException :generated if aSupport is null, if methodName is null or if no method corresponding to the signature exists.
	 * Also generated by a specific mapper to indicate a (specific problem).
	 */
	Object getConstructorValue(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException;
	
	/**
	 * Method executeConstructor. This method performs the effective task associated to the method execute() of a constructor.
	 * @param aClass : the class for the constructor
	 * @param paramObjects : the ordered values of the method parameters
	 * @param typeNames : the ordered types of the method parameters.
	 * @param aContext : the mapping context.
	 * @throws KilimException :generated if aSupport is null, if methodName is null or if no method corresponding to the signature exists.
	 * Also generated by a specific mapper to indicate a (specific problem).
	 */
	void executeConstructor(Class aClass, Object[] paramObjects, String[] typeNames, MappingContext aContext) throws KilimException;			
	
	/**
	 * Method getPropertyValue. This method performs the effective task associated to the method getValue() of a property.
	 * @param aValue : the property identifier.
	 * @param aContext : the mapping context.
	 * @return Object : the value returned by getValue()
	 * @throws KilimException : generated if aValue is null. Also generated by a specific mapper to indicate a (specific problem).
	 */
	Object getPropertyValue(Object aValue, MappingContext aContext) throws KilimException;
		
	/**
	 * Method getClassSourceValue. This method performs the effective task associated to the method getValue() of a ClassSource.
	 * @param aClassName : the name of the class
	 * @param aContext : the mapping context
	 * @return Object : the value returned by getValue() :  generally the corresponding Class object.
	 * @throws KilimException : generated if aClassName is null or if no Class object can be found. 
	 * Also generated by a specific mapper to indicate a (specific problem).
	 */
	Object getClassValue(String aClassName, MappingContext aContext) throws KilimException;

	/**
	 * Method getExternalValue. This method performs the effective task associated to the method getValue() of a external value.
	 * This method exists since the current implementation does not treat external values as properties. It should probably disappear in 
	 * the next release.
	 * @param value : the external reference identifier.
	 * @param aContext : the mapping context.
	 * @return Object :the value returned by getValue() : generally a reference to the object designated by the external value.
	 * @throws KilimException : generated if value is null. Also generated by a specific mapper to indicate a (specific problem).
	 */
	Object getExternalValue(Object value, MappingContext aContext)  throws KilimException;
		
	/**
	 * Method getNullElementValue. This method performs the effective task associated to the method getValue() of a null.
	 * This method exists since the current implementation does not treat null as a propertiy (null may be associated to a transformer).
	 * @param aContext : the mapping context.
	 * @return Object : the value returned by getValue()
	 * @throws KilimException : generated by a specific mapper to indicate a (specific problem).
	 */
	Object getNullElementValue(MappingContext aContext) throws KilimException;

	/**
	 * Method executeNullElement. This method performs the effective task associated to the method execute() of a null.
	 * @param aContext : the mapping context
	 * @throws KilimException : generated by a specific mapper to indicate a (specific problem).
	 */
	void executeNullElement(MappingContext aContext) throws KilimException;
	
	/**
	 * Method getEventSourceValue.....
	 * @param aContext : the mapping context.
	 * @return Object : 
	 * @throws KilimException : generated by a specific mapper to indicate a (specific problem).
	 */
	Object getEventSourceValue(MappingContext aContext) throws KilimException;

}