/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.objectweb.kilim.model.services;

import java.util.Iterator;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.model.Component;
import org.objectweb.kilim.model.ComponentElement;

/**
 * This interface defines the method to be used when dealing which naming contexts.
 * @author horn
 */
public interface NamingContext {
	
	/**
	 * Naming contexts follow a tree-like structure. This method returns the parent context of the current context.
	 * It returns null when applied to the root context.
	 * @return ComponentFactory
	 */
	NamingContext getParentNamingContext();
	
	/**
	 * sets a new parent naming context.
	 * @param aContext : the parent context. Is null for making thje current context a root context.
	 */
	void setParentNamingContext(NamingContext aContext);
	
	/**
	 * adds a child naming context.
	 * @param aName : the local name of the naming context.
	 * @param aContext : the naming context
	 * @throws KilimException : generated if aName is null, if aName is already used, if aContext is null.
	 */
	void addChildNamingContext(String aName, NamingContext aContext) throws KilimException;
	
	/**
	 * removes a child naming context.
	 * @param aName : the name of the naming context to be removed.
	 * @throws KilimException : generated if aName is null or unknown.
	 */
	void removeChildNamingContext(String aName) throws KilimException;
	
	/**
	 * returns as an iterator the names of child naming contexts.
	 * @return Iterator
	 */
	Iterator getChildNamingContexts();
	
	/**
	 * A map of external references can be associated to each naming context. This method sets an ExternalReferenceMap.
	 * @param xReferences : the external references map.
	 */
	void setExternalReferences(ExternalValueReferences xReferences);
	
	/**
	 * gets the ExternalReferenceMap associated to the naming context.
	 * @return HashMap
	 */
	ExternalValueReferences getExternalReferenceMap();
	
	/**
	 * returns the qualified name of the context.
	 * @return String
	 */
	String getQualifiedName();
	
	/**
	 * returns the local name of the context.
	 * @return String
	 */
	String getLocalName();
	
	/**
	 * adds a new lname in the naming context.
	 * @param aName : the name to be bound.
	 * @param aElement : the element.
	 * @throws KilimException : generated if aName is null or is already known.
	 */
	void addBoundName(String aName, ComponentElement aElement) throws KilimException;
	
	/**
	 * removes a name from the naming context.
	 * @param aName : the name of the context to remove.
	 * @throws KilimException : generated if aName is null or unknown in the naming context.
	 */
	void removeBoundName(String aName) throws KilimException;
		
	/**
	 * returns as an iterator all names known in the naming context.
	 * @return Iterator
	 */
	Iterator getBoundNames();
	
	/**
	 * returns the element associated to a name.
	 * @param aName : the name to be resolved.
	 * @param aOrigin : the component from which the resolve request is performed (required for printing error messages)
	 * @return ComponentElement
	 * @throws KilimException : generated if aName is null or unknown.
	 */
	ComponentElement resolveReference(String aName, Component aOrigin) throws KilimException;
}