/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.xml.parsers.ParserConfigurationException;
import org.objectweb.kilim.KilimException;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

/**
 * @author dutoo, horn
 * Implementation of {@link org.objectweb.kilim.repository.ParserErrorHandler}
 * logging exceptions in an output print stream.
 */
public class BasicParserErrorHandler implements ParserErrorHandler {
	private PrintStream out;
	private static final String BASE_HEADER = "Kilim Parser v1.0 ";
	private String header = BASE_HEADER;
	private String templateName;
	
	/**
	 * Creates a new BasicParserErrorHandler logging errors to System.err.
	 */
	public BasicParserErrorHandler() {
		out = System.err;
	}
	
	/**
	 * Creates a new BasicParserErrorHandler logging errors to the given stream.
	 * @param os an output stream
	 */
	public BasicParserErrorHandler(OutputStream os) {
		out = new PrintStream(os);
	}
	
	/**
	 * Method setTemplateName.
	 * @param aName : the name of the template being parsed.
	 */
	public void setTemplateName(String aName) {
		templateName = aName;
		header = BASE_HEADER + " [ template : " + aName + " ] ";
	}
	
	/**
	 * @see org.objectweb.kilim.repository.ParserErrorHandler#handleIOException(IOException)
	 */
	public void handleIOException(IOException ioex) {
		out.println(header + ioex.getLocalizedMessage());
	}
	
	/**
	 * @see org.objectweb.kilim.repository.ParserErrorHandler#handleSAXException(SAXException)
	 */
	public void handleSAXException(SAXException saxex) {
		if (saxex instanceof SAXParseException) {
			handleFatalSAXParseException((SAXParseException) saxex);
		} else {
			out.println(header + saxex.getLocalizedMessage());
		}
	}
	
	/**
	 * @see org.objectweb.kilim.repository.ParserErrorHandler#handleParserConfigurationException(ParserConfigurationException)
	 */
	public void handleParserConfigurationException(ParserConfigurationException pcex) {
		out.println(header + pcex.getLocalizedMessage());
	}
	
	/**
	 * @see org.objectweb.kilim.repository.ParserErrorHandler#handleWarningSAXParseException(SAXParseException)
	 */
	public void handleWarningSAXParseException(SAXParseException saxpex) {
		out.println(getParseErrorMessage(saxpex, "WARNING"));
	}
	
	/**
	 * @see org.objectweb.kilim.repository.ParserErrorHandler#handleErrorSAXParseException(SAXParseException)
	 */
	public void handleErrorSAXParseException(SAXParseException saxpex) {
		out.println(getParseErrorMessage(saxpex, "ERROR"));
	}
	
	/**
	 * @see org.objectweb.kilim.repository.ParserErrorHandler#handleFatalSAXParseException(SAXParseException)
	 */
	public void handleFatalSAXParseException(SAXParseException saxpex) {
		out.println(getParseErrorMessage(saxpex, "FATAL"));
	}
	
	/**
	* @see org.objectweb.kilim.repository.ParserErrorHandler#handleKilimException(KilimException)
	*/
	public void handleKilimException(KilimException ex) {
		//out.println(header + "Bad property typing of value " + ptex.getValue() + " in template " + ptex.getContext());
		out.println("Kilim Exception " + ex);
	}
	
	private String getParseErrorMessage(SAXParseException saxex, String level) {
		StringBuffer msg = new StringBuffer(header + level);
		int line = saxex.getLineNumber();
		int col = saxex.getColumnNumber();
		String publicId = saxex.getPublicId();
		String systemId = saxex.getSystemId();
		Exception embedded = saxex.getException();
		if (line > -1) {
			msg.append(" at line " + line);
		}
		if (col > -1) {
			msg.append(" at column " + col);
		}
		if (publicId != null) {
			msg.append(" at public id " + publicId);
		}
		if (systemId != null) {
			msg.append(" at system id " + systemId);
		}
		if (embedded != null) {
			msg.append(" with embedded exception " + embedded);
		}
		msg.append(" : " + saxex.getLocalizedMessage());
		
		StringWriter strng_wrtr = new StringWriter();
		PrintWriter prt_wrtr = new PrintWriter(strng_wrtr);
		return msg.toString();
	}
}
