/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.objectweb.kilim.description.Instance;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.TemplateDescription;

/**
 * The {@link ResourceMapping} implementation used by the {@link org.objectweb.kilim.repository.TemplateDescriptionParser}.
 * This class is just asimple container used to propagate informations related to template during the parsing process. It contains :
 * 1) a list of already referenced but unparsed templates.
 * 2)a table of known templates whose key is the template name,
 * 3) a table of templates whose key is the instance
 * 4) a table of unknown super template of instances.
 * @author dutoo, horn
 */
public class ResourceMapping {
	private Repository repository;
	private LinkedList unParsedTemplates;
	private HashMap knownTemplates;
	private LinkedList unknownSuperTemplateOfInstances;
	
	/**
	 * Constructor for ResourceMappingImpl.
	 * @param aRepository the repository to be used at template
	 * naming resolution time.
	 */
	public ResourceMapping(Repository aRepository) {
		repository = aRepository;
	}
	
	/**
	 * Method addUnParsedTemplate. This method is invoked when the parser encounters a reference to a template not yet parsed. 
	 * @param name : the name of the template
	 */
	public void addUnparsedTemplate(String name) {
		if (name == null) {
			return;
		}
		
		if (unParsedTemplates == null) {
			unParsedTemplates = new LinkedList();
		}
		if (unParsedTemplates.contains(name)) {
			return;
		}
		unParsedTemplates.add(name);
	}
	
	/**
	 * Method removeUnparsedTemplate. This method is invoked when the parser starts the parsing of a new template.
	 * @param name :
	 */
	public void removeUnparsedTemplate(String name) {
		if (name == null) {
			return;
		}
		if (unParsedTemplates == null) {
			return;
		}
		unParsedTemplates.remove(name);
	}
		
	/**
	 * Method addParsedTemplate.
	 * @param name :
	 * @param aTemplate :
	 */
	public void addKnownTemplate(String name, TemplateDescription aTemplate) {
		if (name == null || aTemplate == null) {
			return;
		}
		if (knownTemplates == null) {
			knownTemplates = new HashMap();
		}
		if (knownTemplates.containsKey(name)) {
			return;
		}
		knownTemplates.put(name, aTemplate);
	}	
	
	/**
	 * Method removeParsedTemplates.
	 * @param name :
	 */
	public void removeKnownTemplate(String name) {
		if (name == null) {
			return;
		}
		if (knownTemplates == null) {
			return;
		}
		knownTemplates.remove(name);
	}
	
	/**
	 * Method getParsedTemplate.
	 * @param name :
	 * @return Template
	 */
	public TemplateDescription getKnownTemplate(String name) {
		if (name == null || knownTemplates == null) {
			return null;
		}
		return (TemplateDescription) knownTemplates.get(name);
	}
		
	/**
	 * Method getNextUnparsedTemplate.
	 * @return String
	 */
	public String getNextUnparsedTemplate() {
		if (unParsedTemplates == null) {
			return null;
		}
		if (unParsedTemplates.size() > 0) {
			String result = (String) unParsedTemplates.getFirst();
			unParsedTemplates.removeFirst();
			return result;
		}
		return null;
	}
		
	/**
	 * Method addUnknownSuperTemplateOfInstance.
	 * @param aInstance :
	 */
	public void addUnknownSuperTemplateOfInstance(Instance aInstance) {
		if (unknownSuperTemplateOfInstances == null) {
			//unknownSuperTemplateOfInstances = new HashMap();
			unknownSuperTemplateOfInstances = new LinkedList();
		}
		//unknownSuperTemplateOfInstances.put(aInstance, aTemplate);
		unknownSuperTemplateOfInstances.addLast(aInstance);
	}
	
	/**
	 * Method getUnknownSuperTemplateOfInstances.
	 * @return Iterator
	 */
	public Iterator getUnknownSuperTemplateOfInstances() {
		if (unknownSuperTemplateOfInstances == null) {
			return KILIM.EMPTY_ITERATOR;
		}

		return unknownSuperTemplateOfInstances.iterator();
	}
}