/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;

import org.objectweb.kilim.KilimConfiguration;
import org.objectweb.kilim.description.Instance;
import org.objectweb.kilim.InternalException;
import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.TemplateDescription;

import org.objectweb.kilim.KilimLoggerFactory;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Implementation of Repository using a ResourceLoader and a kilim xml format Parser.
 * {@see org.objectweb.kilim.repository.Repository}
 * @author dutoo, horn
 */
public class ResourceRepository implements Repository {	
	private static boolean DEBUG_ON = false;	
    private static Logger logger = KilimLoggerFactory.getSingleton().getLogger("org.objectweb.kilim.repository");
	
	private TemplateDescriptionParser tdParser;
	private ResourceLoader resourceLoader;
	private HashMap resourceCache = new HashMap();

	/**
	 * Creates a new ResourceRepository with the given kilim xml format Parser and its ClassLoader as resource loader.
	 */
	public ResourceRepository() {
		tdParser = KilimConfiguration.getTemplateDescriptionParser();
		resourceLoader = KilimConfiguration.getLoader();
	}
	
	/**
	 * Creates a new ResourceRepository with the given kilim xml format Parser and its ClassLoader as resource loader.
	 * @param aParser the kilim xml format parser to be used
	 */
	public ResourceRepository(TemplateDescriptionParser aParser) {
		tdParser = aParser;
		resourceLoader = new ClassLoaderResourceLoader(getClass().getClassLoader());
	}
	
	/**
	 * @see org.objectweb.kilim.repository.Repository#getTemplateDescription(String)
	 */
	public TemplateDescription getTemplateDescription(String resourceName) throws ResourceNotFoundException, KilimException {

		//toBeReturned is the top template to be parsed.	
		TemplateDescription toBeReturned = null;	
		ResourceMapping parsingResult = new ResourceMapping(this);
		toBeReturned = getSingleDescription(resourceName, parsingResult);

		if (DEBUG_ON && logger.isLoggable(BasicLevel.DEBUG)) {
      		logger.log(BasicLevel.DEBUG, "Starting description of \"" + toBeReturned.getName() + "\" ==============================");
		}
		//The repository launches the parser on every ".kilim" files, as long as a referenced template is still unparsed.
		String nextTemplateName;
		while ((nextTemplateName = parsingResult.getNextUnparsedTemplate()) != null) {
			getSingleDescription(nextTemplateName, parsingResult);
		}
		
		//The repository resolves the "super" template references in <instance> tags.
		Iterator iter = parsingResult.getUnknownSuperTemplateOfInstances();

		while (iter.hasNext()) {
			Instance instance = (Instance) iter.next();
			TemplateDescription contain = instance.getContainingTemplate();
			if (contain == null) {
				throw new InternalException("BUG dans getTemplateDescription de ResourceRepository (1) " + instance.getLocalName());
			}
			TemplateDescription scontain = contain.getSuperTemplate();
			if (scontain == null) {
				throw new InternalException("BUG dans getTemplateDescription de ResourceRepository (2) " + instance.getLocalName()  + " defined in template " + instance.getContainingTemplate());
			}
			Instance instance1 = scontain.getInstance(instance.getLocalName(), false);
			if (instance1 == null) {
				throw new InternalException("BUG dans getTemplateDescription de ResourceRepository (3) " + instance.getLocalName()  + " defined in template " + instance.getContainingTemplate());
			}
			
			TemplateDescription temp1 = instance1.getTemplate();
			TemplateDescription temp = instance.getTemplate();
			if (temp == null) {
				instance.setTemplate(temp1);
			}	else {
				temp.setSuperTemplate(temp1);
			}
		}	
		
		if (DEBUG_ON && logger.isLoggable(BasicLevel.DEBUG)) {
      		logger.log(BasicLevel.DEBUG, "State of " + toBeReturned + " ================================= ");
		}

		if (DEBUG_ON && logger.isLoggable(BasicLevel.DEBUG)) {
      			logger.log(BasicLevel.DEBUG, "End of description for \"" + toBeReturned.getName() + "\" ==============================");
		}
		
		return toBeReturned;
	}
	
	/**
	 * Method getSingleDescription. This method parses a single ".kilim" file. It updates the ResourceMapping object
	 * received as a parameter by adding new unknown templates, new "super" references, etc ...
	 * @param resourceName : the name of the template to be parsed
	 * @param parsingResult : the ResourceMapping object used to store the parsing results to be used during next iterations.
	 * @return TemplateDescription : the internal representation of the template described in the resource.
	 * @throws ResourceNotFoundException
	 */
	protected TemplateDescription getSingleDescription(String resourceName, ResourceMapping parsingResult) throws ResourceNotFoundException {
		
		if (DEBUG_ON && logger.isLoggable(BasicLevel.DEBUG)) {
      			logger.log(BasicLevel.DEBUG, "Getting Single Template Description : " + resourceName);
		}
		InputStream is = resourceLoader.getResource(resourceName + KILIM_FILES_EXTENSION);
			
		if (is == null) {
			return null;
		}
		
		if (DEBUG_ON && logger.isLoggable(BasicLevel.DEBUG)) {
      		logger.log(BasicLevel.DEBUG, "Parsing " + resourceName);
		}
		try {
			TemplateDescription toBeReturned = tdParser.importTemplateDescription(is, parsingResult, resourceName);
			toBeReturned.setResourceLoader(resourceLoader);
			return toBeReturned;
		} finally {
			if (is != null) {
				try {
					is.close();
				} catch (IOException ex) {
					System.err.println("Problem when closing input stream of " + resourceName + KILIM_FILES_EXTENSION);
				}
			}
		}
	}
	
	/**
	 * Sets the repository resourceLoader.
	 * @param aResourceLoader : The resourceLoader to set
	 */
	public void setResourceLoader(ResourceLoader aResourceLoader) {
		resourceLoader = aResourceLoader;
	}
	
	/**
	 * Returns the current repository resourceLoader.
	 * @return ResourceLoader
	 */
	public ResourceLoader getResourceLoader() {
		return resourceLoader;
	}
}