/**
 * Copyright (C) 2002 Kelua SA
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.objectweb.kilim.repository;

import java.io.IOException;
import java.io.InputStream;
import java.util.Stack;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.objectweb.kilim.KilimException;
import org.objectweb.kilim.description.Arity;
import org.objectweb.kilim.description.ArraySource;
import org.objectweb.kilim.description.BasicElement;
import org.objectweb.kilim.description.BasicNamedElement;
import org.objectweb.kilim.description.BindTrigger;
import org.objectweb.kilim.description.Binding;
import org.objectweb.kilim.description.ClassSource;
import org.objectweb.kilim.description.EventSource;
import org.objectweb.kilim.description.Instance;
import org.objectweb.kilim.description.KILIM;
import org.objectweb.kilim.description.NullElement;
import org.objectweb.kilim.description.Parameter;
import org.objectweb.kilim.description.Plug;
import org.objectweb.kilim.description.Port;
import org.objectweb.kilim.description.Property;
import org.objectweb.kilim.description.Provider;
import org.objectweb.kilim.description.Reference;
import org.objectweb.kilim.description.Slot;
import org.objectweb.kilim.description.TemplateDescription;
import org.objectweb.kilim.description.TemplateElementImpl;
import org.objectweb.kilim.description.TpAccessor;
import org.objectweb.kilim.description.TpConstructor;
import org.objectweb.kilim.description.TpGetter;
import org.objectweb.kilim.description.TpMethod;
import org.objectweb.kilim.description.TpSetter;
import org.objectweb.kilim.description.Transformer;
import org.objectweb.kilim.description.Trigger;
import org.objectweb.kilim.description.UnBindTrigger;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author dutoo, horn
 * Kilim xml format parser. Uses a JAXP SAXParser. Validates the kilim xml DTD
 * (which is to be found in a ResourceLoader) and logs its error and warnings
 * to a ParserErrorHandler.
 */
public class TemplateDescriptionParser {	
	private static final String CLASSPATH_URI_PREFIX = "classpath://";
	private static final String SUPER_ID = "super";

	private ResourceLoader resourceLoader;
	private SAXParser saxParser;
	private ParserErrorHandler errorHandler;
	//private ResourceBundle parsingMessages = ResourceBundle.getBundle("ParsingMessages");
	private String currentTemplateName;	
		
	/**
	 * Creates a new TemplateDescriptionParser using the JAXP SAX factory's new
	 * default parser, a new BasicErrorHandler and loads DTDs from a new
	 * ClassLoaderResourceLoader.<P>
	 * May log SAXException on parser creation exception
	 * and. ParserConfigurationException if DTD validation is impossible
	 * @param isValidating if true, requires the parsed xml template to be validated
	 * against the kilim_template.dtd, which has therefore to be in the classpath.
	 * NB: the template parser behaviour on parsing error is undetermined if there is
	 * no template DTD validation.
	 */
	public TemplateDescriptionParser(boolean isValidating) {
		errorHandler = new BasicParserErrorHandler(System.err);
		resourceLoader = new ClassLoaderResourceLoader(getClass().getClassLoader());
		try {
			SAXParserFactory spf = SAXParserFactory.newInstance();
			// if isValidating is true, let's require the parsed template to be validated against the xml DTD:
			spf.setValidating(isValidating);
			saxParser = spf.newSAXParser();
		} catch (SAXException saxex) {
			errorHandler.handleSAXException(saxex);
		} catch (ParserConfigurationException pcex) {
			errorHandler.handleParserConfigurationException(pcex);
		}
	}
	
	/**
	 * Creates a new TemplateDescriptionParser using the given custom template SAX parser.
	 * @param aSaxParser the custom kilim xml format parser
	 * @param anErrorHandler to be used for model & parsing errors handling delegation
	 * @param aResourceLoader where the DTDs are to be found
	 * NB: the template parser behaviour on parsing error is undetermined if the given SAX
	 * parser doesn't validate template DTD.
	 */
	public TemplateDescriptionParser(SAXParser aSaxParser, ParserErrorHandler anErrorHandler, ResourceLoader aResourceLoader) {
		saxParser = aSaxParser;
		errorHandler = anErrorHandler;
		resourceLoader = aResourceLoader;
	}
	
	/**
	 * Method importTemplateDescription.
	 * @param is kilim xml format data
	 * @param aParsingResult holds the template dependancies & naming to be solved
	 * @param aName is the template resource name.
	 * @return the parsed Template (its template naming is not resolved yet)
	 */
	public TemplateDescription importTemplateDescription(InputStream is, ResourceMapping aParsingResult, String aName) {
		KilimSaxHandler handler = new KilimSaxHandler(aParsingResult);
		currentTemplateName = aName;
		
		try {
			errorHandler.setTemplateName(aName);
			saxParser.parse(is, handler);
		} catch (SAXException saxex) {
			errorHandler.handleSAXException(saxex);
		} catch (IOException ioex) {
			errorHandler.handleIOException(ioex);
		}
		return handler.getParsedTemplate();
	}
	
	/**
	 * This private class is just a data container used to store information on the stack. The "parsingPhase" field
	 * indicates the nature of the parsed element, objectKind the nature of the stored object and object the reference
	 * to the information itself.
	 */
	private class ContextData {
		int parsingPhase;
		int objectKind;
		Object object;
		
		/**
		 * Method ContextData.
		 * @param aPhase : the parsing phase (which identifies unambiguously to the parsed element)
		 * @param aKind : the kind of the stored object.
		 * @param aObject : the reference to the object.
		 */
		public ContextData(int aPhase, int aKind, Object aObject) {
			parsingPhase = aPhase;
			objectKind = aKind;
			object = aObject;
		}
	}
	
	private class KilimSaxHandler extends DefaultHandler {
		private static final int PARSING_TEMPLATE = 1;
		private static final int PARSING_INSTANCE = 2;
		private static final int PARSING_PORT = 3;
		private static final int PARSING_BINDING = 4;
		private static final int PARSING_PROPERTY = 5;
		private static final int PARSING_ARRAY = 6;
		private static final int PARSING_TRIGGER = 7;
		private static final int PARSING_PROVIDER = 8;
		private static final int PARSING_TRANSFORMER = 9;
		private static final int PARSING_METHOD = 10;
		private static final int PARSING_PARAM = 11;
		private static final int PARSING_ACCESSOR = 12;
		private static final int PARSING_NEW = 13;
		private static final int PARSING_SUPPORT = 14;
		private static final int PARSING_CLASS = 15;
		private static final int PARSING_VALUE = 16;
		private static final int PARSING_SLOT = 16;
		private static final int PARSING_PLUG = 16;
		
		private Locator locator = null;
		private Stack contexts = new Stack();
		private Stack templates = new Stack();
		private Stack instances = new Stack();
		
		private Property currentProperty;
		private StringBuffer currentCDATA;
		private String parentElement;
		
		private TemplateDescription parsedTemplate;
		private ResourceMapping parsingResult;
	
		private boolean isFirstTemplate = true;
		private boolean parsingTrigger;
		private boolean parsingTransformer;
		private int arrayCount;
				
		/**
		 * Method KilimSaxHandler.
		 * @param aParsingResult :the ResourceMapping object used to store and retrieve all parsing result
		 * to be used during next iterations.
		 */
		public KilimSaxHandler(ResourceMapping aParsingResult) {
			parsingResult = aParsingResult;
		}
		
		/**
		 * Method getParsedTemplate.
		 * @return Template : the template described in the parsed resource.
		 */
		public TemplateDescription getParsedTemplate() {
			return parsedTemplate;
		}
		
		/**
		 * Resources whose systemId start with "classpath://" are looked for in the resourceLoader
		 * @see org.xml.sax.EntityResolver#resolveEntity(String, String)
		 */
		public InputSource resolveEntity(String publicId, String systemId) throws SAXNotRecognizedException {
			if (systemId.startsWith(CLASSPATH_URI_PREFIX)) {
				String entityName = systemId.substring(CLASSPATH_URI_PREFIX.length());
				try {
					return new InputSource(resourceLoader.getResource(entityName));
				} catch (ResourceNotFoundException rnfex) {
					// can't find entity resource
					throw new SAXNotRecognizedException(rnfex.getLocalizedMessage());
				}
			}
			// requests that the parser open a regular URI connection to the system Id
			return null;
		}
		
		/**
		 * @see org.xml.sax.ContentHandler#setDocumentLocator(Locator)
		 */
		public void setDocumentLocator(Locator aLocator) {
			locator = aLocator;
		}
		
		/**
		 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
		 */
		public void characters(char[] characters, int start, int length) {
			// read characters are only stored when useful (when parsing property and types)
			if (currentCDATA != null) {
					currentCDATA.append(characters, start, length);
			}
		}
		
		public void fatalError(SAXParseException saxex) {
			errorHandler.handleFatalSAXParseException(saxex);
		}
		
		public void error(SAXParseException saxex) {
			errorHandler.handleErrorSAXParseException(saxex);
		}
		
		public void warning(SAXParseException saxex) {
			errorHandler.handleWarningSAXParseException(saxex);
		}
		
		/**
		 * @see org.xml.sax.ContentHandler#startElement(String, String, String, Attributes)
		 */
		public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
			super.startElement(uri, localName, qName, attributes);
			currentCDATA = new StringBuffer();
			char[] qNameBuffer = qName.toCharArray();
			
			//The startElement is the entry point for all methods dealing with a starting tag. This methods analyzes the tag
			//and then delegates to a dedicated method. qNameContent is analyzed in order to avoid an inefficient sequence of 
			//test. 
			switch (qNameBuffer[0]) {
				case 'a' :
				case 'A' :
					startArray(qName, attributes);
					break;
				case 'b' :
				case 'B' :
					startBind(qName, attributes);
					break;
				case 'c' :
				case 'C' :
					switch (qNameBuffer[1]) {
						case 'a' :
						case 'A' :
							startCall(qName, attributes);
							break;
						case 'l' :
						case 'L' :
							startClass(qName, attributes);
							break;
					}
					break;
				case 'e' :
				case 'E' :
					startEventSource(qName, attributes);
					break;
				case 'g' :
				case 'G' :
					startGet(qName, attributes);
					break;
				case 'i' :
				case 'I' :
					startInstance(qName, attributes);
					break;
				case 'n' :
				case 'N' :
					switch(qNameBuffer[1]) {
						case 'a' :
						case 'A' :
							startNameMapping(qName, attributes);
						case 'e' :
						case 'E' :
							startNew(qName, attributes);
							break;
						case 'u' :
						case 'U' :
							startNullProvider(qName, attributes);
							break;
					}
					break;
				case 'p' :
				case 'P' :
					switch (qNameBuffer[3]) {
						case 'a' :
						case 'A' :
							startParam(qName, attributes);
							break;
						case 't' :
						case 'T' :
							startPort(qName, attributes);
							break;
						case 'p' :
						case 'P' :
							startProperty(qName, attributes);
							break;
						case 'r' :
						case 'R' :
							break;
						case 'v' :
						case 'V' :
							if (qNameBuffer.length == 8) {
								startProvider(qName, attributes);
							} else { /*deprecated*/ }	
							break;
						case 'g' :
						case 'G' :
							startPlug(qName, attributes);
							break;
						default :						
					}
					break;
				case 'r' :
				case 'R' :
					startReference(qName, attributes);
					break;
				case 's' :
				case 'S' :
					switch (qNameBuffer[1]) {
						case 'e' :
						case 'E' : 
							startSet(qName, attributes);
							break;
						case 'l' :
						case 'L' :
							startSlot(qName, attributes);
							break;
					}
					break;
				case 't' :
				case 'T' :
					switch (qNameBuffer[2]) {
						case 'm' :
						case 'M' :
							startTemplate(qName, attributes);
							break;
						case 'p' :
						case 'P' :
							startType(qName, attributes);
							break;
						case 'a' :
						case 'A' :
							if (qNameBuffer.length == 11) {
								startTransformer(qName, attributes);
							} else { /*deprecated*/ }	
							break;
						case 'i' :
						case 'I' :
							startTrigger(qName, attributes);
							break;
					}
					break;						
				case 'v' :
				case 'V' :
					startValue(qName, attributes);
					break;
			}
		}
		
		public void endElement(String uri, String localName, String qName) throws SAXException {
			super.endElement(uri, localName, qName);
			char[] qNameBuffer = qName.toCharArray();
			
			//The endElement is the entry point for all methods dealing with a ending tag. This methods analyzes the tag and
			//then delegates to a dedicated method. qNameContent is analyzed in order to avoid an inefficient sequence of 
			//test. 

			switch (qNameBuffer[0]) {
				case 'a' :
				case 'A' :
					endArray(qName);
					break;
				case 'b' :
				case 'B' :
					endBind(qName);
					break;
				case 'c' :
				case 'C' :
					switch (qNameBuffer[1]) {
						case 'a' :
						case 'A' :
							endCall(qName);
							break;
						case 'l' :
						case 'L' :
							endClass(qName);
							break;
					}
					break;
				case 'e' :
				case 'E' :
					endEventSource(qName);
					break;
				case 'g' :
				case 'G' :
					endGet(qName);
					break;
				case 'i' :
				case 'I' :
					endInstance(qName);
					break;
				case 'n' :
				case 'N' :
					switch(qNameBuffer[1]) {
						case 'a' :
						case 'A' :
							endNameMapping(qName);
						case 'e' :
						case 'E' :
							endNew(qName);
							break;
						case 'u' :
						case 'U' :
							endNullProvider(qName);
							break;
					}
					break;
				case 'p' :
				case 'P' :
					switch (qNameBuffer[3]) {
						case 'a' :
						case 'A' :
							endParam(qName);
							break;
						case 'g' :
						case 'G' :
							endPlug(qName);
							break;
						case 't' :
						case 'T' :
							endPort(qName);
							break;
						case 'p' :
						case 'P' :
							endProperty(qName);
							break;
						case 'v' :
						case 'V' :
							endProvider(qName);
							break;
					}
					break;
				case 'r' :
				case 'R' :
					endReference(qName);
					break;
				case 's' :
				case 'S' :
					switch (qNameBuffer[1]) {
						case 'e' :
						case 'E' :
							endSet(qName);
							break;
						case 'l' :
						case 'L' :
							endSlot(qName);
							break;
					}
					break;
				case 't' :
				case 'T' :
					switch (qNameBuffer[2]) {
						case 'm' :
						case 'M' :
							endTemplate(qName);
							break;
						case 'p' :
						case 'P' :
							endType(qName);
							break;
						case 'a' :
						case 'A' :
							endTransformer(qName);
							break;
						case 'i' :
						case 'I' :
							endTrigger(qName);
							break;
					}
					break;
				case 'v' :
				case 'V' :
					endValue(qName);
					break;
			}
		}
		
		/**
		 * Method used to handle the starting <template> tag. It indicates the top template described in the
		 * current resource. This method is therefore called once at the beginning of the parsing process.
		 */
				
		private void startTemplate(String qN, Attributes attributes) {
			TemplateDescription template = null;
			TemplateDescription superTemplate = null;
			
			String name = attributes.getValue("name");
			if (!name.equals(currentTemplateName)) {
				System.err.println("WARNING : template name " + name + " differs from resource template name " + currentTemplateName);
			}
			
			String superTemplateName = attributes.getValue("extends");
			
			try {
				//The parser creates a TemplateDescription object when it encounters a reference to a new unknown template.
				//This TemplateDescription object is stored in parsingResult. As a consequence parsingResult is looked up 
				//in order to check whether the TemplateDescription object corresponding to the current template has been
				//registered in parsingResult.
				template = parsingResult.getKnownTemplate(currentTemplateName);
				
				//The current template has not been previously registered in parsingResult. The TemplateDescription object 
				//is thus created and registered. The current template name is removed from the unparsedTemplate set.
				if (template == null) {
					template = new TemplateDescription(currentTemplateName);
					parsingResult.addKnownTemplate(currentTemplateName, template);
					parsingResult.removeUnparsedTemplate(currentTemplateName);	
				}
								
				//The current template is declared as an extension of another template. 
				if (superTemplateName != null) {
					//parsingResult is looked up in order to check whether the super template is already known.
					superTemplate = parsingResult.getKnownTemplate(superTemplateName);
					if (superTemplate == null) {
						//the super template is unknown. A new TemplateDescription object is thus created and
						//registered in resultMapping as a known template.
						//The name of the super template is added to the set of unparsed templates.
						superTemplate = new TemplateDescription(superTemplateName);
						parsingResult.addUnparsedTemplate(superTemplateName);
						parsingResult.addKnownTemplate(superTemplateName, superTemplate);						
						template.setSuperTemplate(superTemplate);
					} else {
						//The superTemplate is known and is set as the super template of the current template.
						template.setSuperTemplate(superTemplate);
					}
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}

			//The current template is pushed in the templates stack and in the contexts stack.
			templates.push(template);
			contexts.push(new ContextData(PARSING_TEMPLATE, 0, template));
			if (isFirstTemplate) {
				//The template to be parsed is the top template and is thus the one to be returned.
				parsedTemplate = template;
				isFirstTemplate = false;
				try {
					//An instance is created corresponding to the top template.
					Instance topInstance = new Instance("", KILIM.PUBLIC, template, null);
					instances.push(topInstance);
				} catch (KilimException exc) {
					System.err.println("BUG : error when creating top instance of " + template.getName());
				}
			}
		}
		
		/**
		 * Method used to handle the ending </template> tag. it corresponds to the starting tag encountered at the
		 * beginning of the resource.This method is therefore called once at the end of the parsing process.This
		 * method checks the stack state and invokes the method that performrs the final checks on the parsed
		 * template.
		 */
		private void endTemplate(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			if (phase != PARSING_TEMPLATE) {
				displayContextStack("context stack problem detected in endTemplate", contextData);
			}
			templates.pop();
		}	
		
		private void startProperty(String qName, Attributes attributes) {
			String name = attributes.getValue("name");
			int status = getStatus(attributes.getValue("status"), "protected");
			int typeKind = getTypeKind(attributes.getValue("type"));

			Property property = null;
			try {			
				if (name != null && arrayCount == 0) {
					property = new Property(name, status, typeKind, getCurrentTemplate());
					getCurrentTemplate().addLocalProperty(property);
				} else {
					if (arrayCount > 0) {
						status = KILIM.PRIVATE;
					}
					if (name != null) {
						property = new Property("%anonymous" + name, status, typeKind, getCurrentTemplate());
					} else {
						property = new Property("%anonymous" + KILIM.getUniqueIndex(), status, typeKind, getCurrentTemplate());
					}
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			//just a hack : typeKind is stored in the objectKind field of contextData (should be changed)								
			contexts.push(new ContextData(PARSING_PROPERTY, typeKind, property));
			currentCDATA = new StringBuffer();
		}
		
		private void endProperty(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_PROPERTY) {
				displayContextStack("context stack problem detected in endProperty", contextData);
			}

			Property property = (Property) contextData.object;			
			//just a hack : typeKind is stored in the contextData in the objectKind field
			int typeKind = contextData.objectKind;		
			String svalue = currentCDATA.toString();
			Object value = getValue(getCDATA(svalue), typeKind);

			try {
				if (value != null) {
					property.setValue(value);
				}
				ContextData newContext = (ContextData) contexts.peek();
				phase = newContext.parsingPhase;
						
				switch(phase) {	
					case PARSING_PROVIDER :
						Provider provider = (Provider) newContext.object;
						provider.setSource(property);
						newContext.object = provider;
						break;
					case PARSING_PARAM :
						Parameter param = (Parameter) newContext.object;
						param.setTarget(property);
						break;
					case PARSING_ARRAY :
						ArraySource array = (ArraySource) newContext.object;
						array.addElement(property);
						break;
					case PARSING_BINDING :
						Binding binding = (Binding) newContext.object;
						binding.bindProvider(property);
						break;
					case PARSING_SUPPORT :
					case PARSING_VALUE :
						newContext.object = property;
						break;
					case PARSING_TRANSFORMER :
						Transformer transformer = (Transformer) newContext.object;
						transformer.setAction(property);
						newContext.object = transformer;
						break;
					case PARSING_TRIGGER :
						Trigger trig = (Trigger) newContext.object;
						trig.addTransformer(property);
						break;
					default :
						//displayContextStack("context stack problem (3) detected in endProperty", newContext);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			currentCDATA = null;			
		}
		
		private void startProvider(String qName, Attributes attributes) {
			Provider provider = null;
			String name = attributes.getValue("name");
			int status = getStatus(attributes.getValue("status"), "protected");
			try {
				if (name != null) {
					provider = new Provider(name, status, getCurrentTemplate());
				} else {
					logParseException("no null name allowed in a provider tag");
					provider = new Provider("unknown", status, getCurrentTemplate());
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			contexts.push(new ContextData(PARSING_PROVIDER, 0, provider));
		}
		
		private void endProvider(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_PROVIDER) {
				displayContextStack("context stack problem detected in endProvider", contextData);
			}
			try {
				getCurrentTemplate().addLocalProvider((BasicNamedElement) contextData.object);
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void startTransformer(String qName, Attributes attributes) {
			Transformer transformer = null;			
			String name = attributes.getValue("name");
			int status = getStatus(attributes.getValue("status"), "protected");
			try {
				if (name != null) {
					transformer = new Transformer(name, status, getCurrentTemplate());
				} else {
					logParseException("illegal null name in a transformer tag");
					transformer = new Transformer("unknown", status, getCurrentTemplate());
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			contexts.push(new ContextData(PARSING_TRANSFORMER, 0, transformer));
			parsingTransformer = true;
		}
		
		private void endTransformer(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_TRANSFORMER) {
				displayContextStack("context stack problem detected in endTransformer", contextData);
			}
			try {
				getCurrentTemplate().addLocalTransformer((BasicNamedElement) contextData.object);
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			
			parsingTransformer = false;
		}
		
		private void startGet(String qName, Attributes attributes) {
			String field = attributes.getValue("field");
			boolean isStatic = "true".equalsIgnoreCase(attributes.getValue("static"));
			try {
				TpAccessor fieldAction = new TpGetter(null, field, isStatic, getCurrentTemplate());
				contexts.push(new ContextData(PARSING_ACCESSOR, 0, fieldAction));
				contexts.push(new ContextData(PARSING_SUPPORT, PARSING_ACCESSOR, null));
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endGet(String qName) {
			ContextData currentContext = (ContextData) contexts.pop();
								
			if (currentContext.parsingPhase != PARSING_SUPPORT) {
				displayContextStack("context stack problem (1) detected in endGet", currentContext);
			}
			
			BasicElement support = (BasicElement) currentContext.object;
			
			currentContext = (ContextData) contexts.pop();
			if (currentContext.parsingPhase != PARSING_ACCESSOR) {
				displayContextStack("context stack problem (2) detected in endGet", currentContext);
			}
			
			TpAccessor fieldSource = (TpAccessor) currentContext.object;
			
			ContextData newContext = (ContextData) contexts.peek();
			int phase = newContext.parsingPhase;
			
			try {
				fieldSource.setSupport(support);		
				switch(phase) {	
					case PARSING_PROVIDER :
						Provider provider = (Provider) newContext.object;
						provider.setSource(fieldSource);
						newContext.object = provider;
						break;
					case PARSING_PARAM :
						Parameter param = (Parameter) newContext.object;
						param.setTarget(fieldSource);
						break;
					case PARSING_ARRAY :
						ArraySource array = (ArraySource) newContext.object;
						array.addElement(fieldSource);
						break;
					case PARSING_BINDING :
						Binding binding = (Binding) newContext.object;
						binding.bindProvider(fieldSource);
						break;
					case PARSING_SUPPORT :
					case PARSING_VALUE :
						newContext.object = fieldSource;
						break;
					case PARSING_TRANSFORMER :
						logParseException("no get is allowed in the definition of a transformer");
						break;
					case PARSING_TRIGGER :
						Trigger trig = (Trigger) newContext.object;
						trig.addTransformer(fieldSource);
						break;
					default :
						displayContextStack("context stack problem (3) detected in endGet", newContext);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void startSet(String qName, Attributes attributes) {
			String field = attributes.getValue("field");
			boolean isStatic = "true".equalsIgnoreCase(attributes.getValue("static"));
			try {
				TpAccessor fieldAction = new TpSetter(null, field, isStatic, getCurrentTemplate());
				contexts.push(new ContextData(PARSING_ACCESSOR, 0, fieldAction));
				contexts.push(new ContextData(PARSING_SUPPORT, PARSING_ACCESSOR, null));
				} catch (KilimException ex) {
					logParseException(ex.toString());
			}
		}
		
		private void endSet(String qName) {
			ContextData currentContext = (ContextData) contexts.pop();
								
			if (currentContext.parsingPhase != PARSING_SUPPORT) {
				displayContextStack("context stack problem (1) detected in endSet", currentContext);
			}
			
			BasicElement support = (BasicElement) currentContext.object;
			
			currentContext = (ContextData) contexts.pop();
			if (currentContext.parsingPhase != PARSING_ACCESSOR) {
				displayContextStack("context stack problem (2) detected in endSet", currentContext);
			}
			
			TpAccessor fieldTransformer = (TpAccessor) currentContext.object;	
			
			ContextData newContext = (ContextData) contexts.peek();
			int phase = newContext.parsingPhase;
			try {
				fieldTransformer.setSupport(support);	
				if (phase == PARSING_TRANSFORMER) {
					Transformer transformer = (Transformer) newContext.object;
					transformer.setAction(fieldTransformer);
				} else {
					if (phase == PARSING_TRIGGER) {
						Trigger trig = (Trigger) newContext.object;
						trig.addTransformer(fieldTransformer);
					} else {
						displayContextStack("context stack problem (3) detected in endSet", newContext);
					}
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void startNew(String qName, Attributes attributes) {
			try {
				TpConstructor ctorAction = new TpConstructor(null, getCurrentTemplate());
				contexts.push(new ContextData(PARSING_NEW, 0, ctorAction));
				//the following line is useless. It is introduced for regularity reasons
				contexts.push(new ContextData(PARSING_SUPPORT, 0, null));
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endNew(String qName) {
			ContextData currentContext = (ContextData) contexts.pop();
								
			if (currentContext.parsingPhase != PARSING_SUPPORT) {
				displayContextStack("context stack problem (1) detected in endNew", currentContext);
			}
					
			BasicElement support = (BasicElement) currentContext.object;
			currentContext = (ContextData) contexts.pop();			
					
			if (currentContext.parsingPhase != PARSING_NEW) {
				displayContextStack("context stack problem (2) detected in endNew", currentContext);
			}
			
			TpConstructor ctorAction = (TpConstructor) currentContext.object;
			
			ContextData newContext = (ContextData) contexts.peek();
			int phase = newContext.parsingPhase;
			
			try {
				ctorAction.setSupport(support);
				switch(phase) {
					case PARSING_TRANSFORMER :
						Transformer transformer = (Transformer) newContext.object;
						transformer.setAction(ctorAction);
						break;
					case PARSING_PROVIDER :
						Provider provider = (Provider) newContext.object;
						provider.setSource(ctorAction);
						break;
					case PARSING_PARAM :
					 	Parameter param = (Parameter) newContext.object;
						param.setTarget(ctorAction);
						break;
					case PARSING_ARRAY :
						ArraySource array = (ArraySource) newContext.object;
						array.addElement(ctorAction);
						break;
					case PARSING_BINDING :
						Binding binding = (Binding) newContext.object;
						binding.bindProvider(ctorAction);
						break;
					case PARSING_SUPPORT :
					case PARSING_VALUE :
						newContext.object = ctorAction;
						break;
					case PARSING_TRIGGER :
						Trigger trig = (Trigger) newContext.object;
						trig.addTransformer(ctorAction);
						break;
					default :
						displayContextStack("context stack problem (3) detected in endNew", newContext);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}	
		
		private void startCall(String qName, Attributes attributes) {
			String method = attributes.getValue("method");
			boolean isStatic = "true".equalsIgnoreCase(attributes.getValue("static"));
			try {
				TpMethod methodAction = new TpMethod(null, method, isStatic, true, true, getCurrentTemplate());
				contexts.push(new ContextData(PARSING_METHOD, 0, methodAction));
				contexts.push(new ContextData(PARSING_SUPPORT, PARSING_METHOD, null));
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}	
		
		private void endCall(String qName) {
			ContextData currentContext = (ContextData) contexts.pop();
								
			if (currentContext.parsingPhase != PARSING_SUPPORT) {
				displayContextStack("context stack problem (1) detected in endCAll", currentContext);
			}
			
			BasicElement support = (BasicElement) currentContext.object;
			
			currentContext = (ContextData) contexts.pop();
			if (currentContext.parsingPhase != PARSING_METHOD) {
				displayContextStack("context stack problem (2) detected in endCall", currentContext);
			}
			
			TpMethod methodAction = (TpMethod) currentContext.object;	
			
			ContextData newContext = (ContextData) contexts.peek();
			int phase = newContext.parsingPhase;

			try {
				methodAction.setSupport(support);
				switch(phase) {
					case PARSING_PROVIDER :
						Provider provider = (Provider) newContext.object;
						provider.setSource(methodAction);
						break;
					case PARSING_TRANSFORMER :
						Transformer transformer = (Transformer) newContext.object;
						transformer.setAction(methodAction);
						break;
					case PARSING_PARAM :
						Parameter param = (Parameter) newContext.object;
						param.setTarget(methodAction);
						break;
					case PARSING_ARRAY :
						ArraySource array = (ArraySource) newContext.object;
						array.addElement(methodAction);
						break;
					case PARSING_BINDING :
						Binding binding = (Binding) newContext.object;
						binding.bindProvider(methodAction);
						break;
					case PARSING_SUPPORT :
					case PARSING_VALUE :
						newContext.object = methodAction;
						break;
					case PARSING_TRIGGER :
						Trigger trig = (Trigger) newContext.object;
						trig.addTransformer(methodAction);
						break;
					default :
						displayContextStack("context stack problem (3) detected in endCall", newContext);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void startArray(String qName, Attributes attributes) {
			arrayCount++;
			String typeName = attributes.getValue("type");
			int typeKind = getTypeKind(typeName);
			
			try {
				ArraySource array;
				if (typeKind == KILIM.OBJECT) {
					array = new ArraySource(typeName, getCurrentTemplate());
				} else {
					array = new ArraySource(KILIM.CLASS_NAME[typeKind], getCurrentTemplate());
				}
				contexts.push(new ContextData(PARSING_ARRAY, 0, array));
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endArray(String qName) {
			arrayCount--;
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_ARRAY) {
				displayContextStack("context stack problem (1) detected in endArray", contextData);
			}
			
			ArraySource array = (ArraySource) contextData.object;			
			contextData = (ContextData) contexts.peek();
			phase = contextData.parsingPhase;
			TemplateElementImpl source = null;

			try {
				switch(phase) {
					case PARSING_PROVIDER :
						TemplateElementImpl element = (TemplateElementImpl) contextData.object;
						contextData.object = array;
						break;
					case PARSING_SUPPORT :
					case PARSING_VALUE :
						contextData.object = array;
						break;
					case PARSING_PARAM :
						Parameter param = (Parameter) contextData.object;
						param.setTarget(array);
						break;
					case PARSING_ARRAY :
						ArraySource array1 = (ArraySource) contextData.object;
						array1.addElement(array);
						break;
					case PARSING_BINDING:
						Binding binding = (Binding) contextData.object;
						binding.bindProvider(array);
						break;
					default :
						displayContextStack("context stack problem (2) detected in endArray", contextData);
						break;
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
	
		private void startClass(String qName, Attributes attributes) {
			String className = attributes.getValue("name");
			if (className == null) {
				logParseException("no null name allowed in class definition");
			}

			ContextData contextData = (ContextData) contexts.peek();
			int phase = contextData.parsingPhase;
			try {
				ClassSource classSource = new ClassSource(className, getCurrentTemplate());
				switch(phase) {
					case PARSING_PROVIDER :
						Provider provider = (Provider) contextData.object;
						provider.setSource(classSource);
						break;
					case PARSING_SUPPORT :
					case PARSING_VALUE :
						contextData.object = classSource;
						break;
					case PARSING_PARAM :
						Parameter param = (Parameter) contextData.object;
						param.setTarget(classSource);
						break;
					case PARSING_ARRAY :
						ArraySource array = (ArraySource) contextData.object;
						array.addElement(classSource);
						break;
					case PARSING_BINDING :
						Binding binding = (Binding) contextData.object;
						binding.bindProvider(classSource);
						break;
					default :
						displayContextStack("context stack problem detected in startClass", contextData);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endClass(String qName) { }
		
		private void startParam(String localName, Attributes attributes) {
			String paramName = attributes.getValue("name");
			String paramType = attributes.getValue("type");
			contexts.push(new ContextData(PARSING_PARAM, 0, new Parameter(paramName, paramType, null)));
		}
		
		private void endParam(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_PARAM) {
				displayContextStack("context stack problem (1) detected in endParam", contextData);
			}			
			
			Parameter parameter = (Parameter) contextData.object;
			contextData = (ContextData) contexts.peek();
			phase = contextData.parsingPhase;
			
			if (phase != PARSING_SUPPORT) {
				displayContextStack("context stack problem (2) detected in endParam", contextData);
			}	
			
			contextData = (ContextData) contexts.elementAt(contexts.size() - 2);
			phase = contextData.parsingPhase;
			try {
				switch(phase) {
					case PARSING_METHOD :
					case PARSING_NEW :
						TpMethod methodAction = (TpMethod) contextData.object;
						methodAction.addParameter(parameter);
						break;
					default :
						displayContextStack("context stack problem (3) detected in endParam", contextData);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void startValue(String qName, Attributes attributes) {
			contexts.push(new ContextData(PARSING_VALUE, 0, null));
		}

		private void endValue(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_VALUE) {
				displayContextStack("context stack problem (1) detected in endValue", contextData);
			}
			
			BasicElement provider = (BasicElement) contextData.object;
			
			contextData = (ContextData) contexts.peek();
			phase = contextData.parsingPhase;
			
			if (phase != PARSING_SUPPORT) {
				displayContextStack("context stack problem (2) detected in endValue", contextData);
			}	
			
			contextData = (ContextData) contexts.elementAt(contexts.size() - 2);
			phase = contextData.parsingPhase;	
			
			switch(phase) {
				case PARSING_ACCESSOR :
					TpSetter setter = (TpSetter) contextData.object;
					setter.setValue(provider);
					break;
				default :
					displayContextStack("context stack problem (3) detected in endValue", contextData);
					break;	
			}
		}
		
		private Reference getRef(String aName, boolean isP, boolean isT) throws KilimException {
			Reference ref = getCurrentTemplate().getReference(aName, true);
			if (ref == null) {
				ref = new Reference(aName, getCurrentTemplate(), isP, isT);
			} else {
				if (!ref.providesValue()) {
					throw new KilimException("attempt to reuse reference " + aName + "as a value provider in template " + getCurrentTemplate());
				}
			}
			return ref;
		}
		
		private void startReference(String qName, Attributes attributes) {
			String targetName = attributes.getValue("target");
			if (targetName == null) {
				logParseException("no null name allowed in provider reference definition");
			}		
		
			ContextData contextData = (ContextData) contexts.peek();
			int phase = contextData.parsingPhase;
			Reference ref = null;	
			
			try {			
				switch(phase) {
					case PARSING_PROVIDER :
						Provider provider = (Provider) contextData.object;
						ref = getRef(targetName, true, false);
						provider.setSource(ref);
						getCurrentTemplate().addProviderReference(ref);
						break;
					case PARSING_SUPPORT :
						ref = getRef(targetName, true, false);					
						contextData.object = ref;
						getCurrentTemplate().addProviderReference(ref);
						break;
					case PARSING_PARAM :
						Parameter param = (Parameter) contextData.object;
						ref = getRef(targetName, true, false);
						param.setTarget(ref);
						getCurrentTemplate().addProviderReference(ref);
						break;
					case PARSING_ARRAY :
						ArraySource array = (ArraySource) contextData.object;
						ref = getRef(targetName, true, false);
						array.addElement(ref);
						getCurrentTemplate().addProviderReference(ref);
						break;
					case PARSING_BINDING :
						Binding binding = (Binding) contextData.object;
						ref = getRef(targetName, true, false);
						binding.bindProvider(ref);
						getCurrentTemplate().addProviderReference(ref);
						break;
					case PARSING_TRANSFORMER :
						Transformer transformer = (Transformer) contextData.object;
						ref = getRef(targetName, false, true);				
						transformer.setAction(ref);
						getCurrentTemplate().addTransformerReference(ref);
						break;
					case PARSING_TRIGGER :
						Trigger trigger = (Trigger) contextData.object;
						ref = getRef(targetName, false, true);
						trigger.addTransformer(ref);
						getCurrentTemplate().addTransformerReference(ref);
						break;
					case PARSING_VALUE :
						ref = getRef(targetName, true, false);
						contextData.object = ref;
						getCurrentTemplate().addProviderReference(ref);
						break;					
					default :
						displayContextStack("context stack problem detected in startReference", contextData);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endReference(String qName) { }
					
		private void startEventSource(String qName, Attributes attributes) {
			if (!parsingTransformer && !parsingTrigger) {
				logParseException("no event source allowed outside a trigger-transformer context");
			}
			
			ContextData contextData = (ContextData) contexts.peek();
			int phase = contextData.parsingPhase;
			try {
				switch(phase) {
					case PARSING_PARAM :
						Parameter param = (Parameter) contextData.object;
						param.setTarget(EventSource.EVENT_SOURCE);
						break;
					case PARSING_SUPPORT :
						contextData.object = EventSource.EVENT_SOURCE;	
						break;	
					case PARSING_ARRAY :
						ArraySource array = (ArraySource)  contextData.object;
						array.addElement(EventSource.EVENT_SOURCE);
						break;
					default :
						displayContextStack("context stack problem detected in startEventTemplateElement", contextData);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endEventSource(String qName) { }
		
		private void startInstance(String qName, Attributes attributes) {
			String name = attributes.getValue("name");
			String templateName = (String) attributes.getValue("template");
			int status = getStatus(attributes.getValue("status"), "protected");
			boolean hasUnknownSuper = false;
			
			TemplateDescription superTemplate = null;
			//TemplateDescription superTemplateForSuper = null;
			if (templateName != null) {
				if ("super".equalsIgnoreCase(templateName)) {
					hasUnknownSuper = true;
				} else {
					superTemplate = (TemplateDescription) parsingResult.getKnownTemplate(templateName);
					if (superTemplate == null) {
						try {
							superTemplate = new TemplateDescription(templateName);
						} catch (KilimException ex) {
							logParseException(ex.toString());
						}
						parsingResult.addUnparsedTemplate(templateName);
						parsingResult.addKnownTemplate(templateName, superTemplate);		
					}	
				}
			}
			try {
				TemplateDescription newTemplate = new TemplateDescription("%anonymous" + KILIM.getUniqueIndex(), superTemplate);
				TemplateDescription contain = getCurrentInstance().getTemplate();
				Instance newInstance = new Instance(name, status, newTemplate, contain);		
				contain.addLocalInstance(newInstance);
				templates.push(newTemplate);
				instances.push(newInstance);	
				contexts.push(new ContextData(PARSING_INSTANCE, 0, newInstance));
				if (hasUnknownSuper) {
					parsingResult.addUnknownSuperTemplateOfInstance(newInstance);
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endInstance(String localName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_INSTANCE) {
				displayContextStack("context stack problem (1) detected in endInstance", contextData);
			}
			
			Instance instance = (Instance) contextData.object;
			TemplateDescription template = instance.getTemplate();
			if (template == null) {
				logParseException("instance " + instance.getLocalName() + " without template");
			}
			try {
				if (!template.containsNewDefinitions()) {
					instance.setTemplate(template.getSuperTemplate());
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			instances.pop();
			templates.pop();
		}
		
		private void startBind(String qName, Attributes attributes) {
			String portName = attributes.getValue("port");
			Binding binding = null;
			try {			
				binding = new Binding(portName, getCurrentTemplate());
			} catch (KilimException ex) {
				try {
					binding = new Binding("error_" + KILIM.getUniqueIndex(), getCurrentTemplate());
				} catch (KilimException ex1) { }
				logParseException(ex.toString());
			}
			contexts.push(new ContextData(PARSING_BINDING, 0, binding));
			try {
				getCurrentTemplate().addLocalBinding(binding);
			} catch (KilimException ex) {
				logParseException(ex.toString());				
			}
		}
		
		private void endBind(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_BINDING) {
				displayContextStack("context stack problem (1) detected in endBind", contextData);
			}
		}
		
		private void startTrigger(String localName, Attributes attributes) {
			String source = attributes.getValue("source");
			String event = attributes.getValue("event");
			// dtd asserts currentPort is not null
			Trigger trigger = null;
			if ("bind".equalsIgnoreCase(event)) {
				trigger = new BindTrigger(source);
			} else if ("unbind".equalsIgnoreCase(event)) {
				trigger = new UnBindTrigger(source);
			}
			parsingTrigger = true;
			contexts.push(new ContextData(PARSING_TRIGGER, 0, trigger));
			try {
				getCurrentTemplate().addLocalTrigger(trigger);
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}			
		}
		
		private void endTrigger(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_TRIGGER) {
				displayContextStack("context stack problem (1) detected in endTrigger", contextData);
			}
			parsingTrigger = false;
		}
		
		private void startPort(String qName, Attributes attributes) {
			Port port = null;
			String name = attributes.getValue("name");
			int status = getStatus(attributes.getValue("status"), "protected");
			String sRole = attributes.getValue("role");
			boolean isProvided = false;
			if (sRole == null || "offered".equalsIgnoreCase(sRole)) {
				isProvided = true;
			}
			
			String sArity = attributes.getValue("arity");
			Arity arity = null;

			try {
				if ("?".equalsIgnoreCase(sArity) || "optional".equalsIgnoreCase(sArity)) {
					arity = Arity.OPTIONAL;
					port = new Port(name, Arity.OPTIONAL, status, isProvided, getCurrentTemplate());
				} else if ("1".equalsIgnoreCase(sArity)) {
					arity = Arity.REQUIRED;
					port = new Port(name, Arity.REQUIRED, status, isProvided, getCurrentTemplate());			
				} else if ("*".equalsIgnoreCase(sArity) || "any".equalsIgnoreCase(sArity)) {
					arity = Arity.COLLECTION;
					port = new Port(name, Arity.COLLECTION, status, isProvided, getCurrentTemplate());
				} else {
					logParseException("unable to parse arity " + sArity);
				}
				//getCurrentTemplate().addLocalPort(port);
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			contexts.push(new ContextData(PARSING_PORT, 0, port));
			return;
		}
		
		private void endPort(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			if (phase != PARSING_PORT) {
				displayContextStack("context stack problem detected in endPort", contextData);
			}
			Port port = (Port) contextData.object;
			
			ContextData newContext = (ContextData) contexts.peek();
			phase = newContext.parsingPhase;
			try {
				switch(phase) {
					case PARSING_SLOT :
						Slot slot = (Slot) newContext.object;
						slot.addPort(port);
						break;	
					default :
						getCurrentTemplate().addLocalPort(port);
						break;
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void startNullProvider(String qName, Attributes attributes) {
			ContextData contextData = (ContextData) contexts.peek();
			int phase = contextData.parsingPhase;
			try {
				NullElement nullElement  = null;
				switch(phase) {
					case PARSING_PROVIDER :
						Provider provider = (Provider) contextData.object;
						nullElement = new NullElement(provider.getLocalName(), provider.getStatus(), true, provider.performsAction(), getCurrentTemplate());
						provider.setSource(nullElement);
						break;
					case PARSING_TRANSFORMER :
						Transformer transformer = (Transformer) contextData.object;
						nullElement = new NullElement(transformer.getLocalName(), transformer.getStatus(), transformer.performsAction(), true, getCurrentTemplate());
						transformer.setAction(nullElement);
						break;
					case PARSING_SUPPORT :
						throw new KilimException("attempt to set null to a support in template  " + getCurrentTemplate());
					case PARSING_VALUE :
						nullElement = new NullElement("%anonymous" + KILIM.getUniqueIndex(), KILIM.PRIVATE, true, false, getCurrentTemplate());
						contextData.object = nullElement;
						break;
					case PARSING_PARAM :
						Parameter param = (Parameter) contextData.object;
						nullElement = new NullElement("%anonymous" + KILIM.getUniqueIndex(), KILIM.PRIVATE, true, false, getCurrentTemplate());
						param.setTarget(nullElement);
						break;
					case PARSING_ARRAY :
						ArraySource array = (ArraySource) contextData.object;
						nullElement = new NullElement("%anonymous" + KILIM.getUniqueIndex(), KILIM.PRIVATE, true, false, getCurrentTemplate());
						array.addElement(nullElement);
						break;
					case PARSING_BINDING :
						Binding binding = (Binding) contextData.object;
						nullElement = new NullElement("%anonymous" + KILIM.getUniqueIndex(), KILIM.PRIVATE, true, false, getCurrentTemplate());
						binding.bindProvider(nullElement);
						break;
					default :
						displayContextStack("context stack problem detected in startClass", contextData);
						break;	
				}
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}

		}
		
		private void endNullProvider(String qName) { }
		
		private void startSlot(String qName, Attributes attributes) {
			String name = attributes.getValue("name");
			int status = getStatus(attributes.getValue("status"), "protected");
			try {
				Slot slot = new Slot(name, status, getCurrentTemplate());
				getCurrentTemplate().addLocalSlot(slot);
				contexts.push(new ContextData(PARSING_SLOT, 0, slot));
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			return;
		}
	
		private void endSlot(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			if (phase != PARSING_SLOT) {
				displayContextStack("context stack problem detected in endSlot", contextData);
			}
		}
		
		private String slotNameInCurrentPlug;
		
		private void startPlug(String qName, Attributes attributes) {
			String instanceName = attributes.getValue("instance");
			String slotName = attributes.getValue("slot");
			slotNameInCurrentPlug = slotName;
			try {		
				Plug plug = new Plug(slotName, instanceName, getCurrentTemplate());
				getCurrentTemplate().addLocalPlug(plug);
				contexts.push(new ContextData(PARSING_SLOT, 0, plug));
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
			
			return;		
		}

		private void endPlug(String qName) {
			ContextData contextData = (ContextData) contexts.pop();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_PLUG) {
				displayContextStack("context stack problem (1) detected in endPlug ", contextData);
			}
		}
		
		private void startNameMapping(String qName, Attributes attributes) {
			ContextData contextData = (ContextData) contexts.peek();
			int phase = contextData.parsingPhase;
			
			if (phase != PARSING_PLUG) {
				displayContextStack("context stack problem (1) detected in startNameParam ", contextData);
			}
			Plug currentPlug = (Plug) contextData.object;
			String external = attributes.getValue("external");
			String internal = attributes.getValue("internal");
			try {
				currentPlug.addNameMapping(external, internal);
				return;
			} catch (KilimException ex) {
				logParseException(ex.toString());
			}
		}
		
		private void endNameMapping(String qName) { }
		
		private void startType(String qName, Attributes attributes) {
			currentCDATA = new StringBuffer();
		}
		
		private void endType(String qName) {
			currentCDATA = null;
		}

		
		/**
		 * Method getType : returns the integer "type" constant (defined in the _Referenceable interface)
		 * corresponding to the type passed as a String.
		 * @param aType
		 * @return int
		 */
		private int getTypeKind(String aType) {
			if ("char".equalsIgnoreCase(aType)) {
				return KILIM.CHAR;
			} else if ("boolean".equalsIgnoreCase(aType)) {
				return KILIM.BOOLEAN;
			} else if ("byte".equalsIgnoreCase(aType)) {
				return KILIM.BYTE;
			} else if ("short".equalsIgnoreCase(aType)) {
				return KILIM.SHORT;
			} else if ("int".equalsIgnoreCase(aType)) {
				return KILIM.INT;
			} else if ("long".equalsIgnoreCase(aType)) {
				return KILIM.LONG;
			} else if ("float".equalsIgnoreCase(aType)) {
				return KILIM.FLOAT;
			} else if ("double".equalsIgnoreCase(aType)) {
				return KILIM.DOUBLE;
			} else if ("string".equalsIgnoreCase(aType)) {
				return KILIM.STRING;
			}
			return KILIM.OBJECT;
		}
		
		/**
		 * Method getValue : returns a java wrapper containing the value of a primitive java type passed as a String.
		 * @param aValue
		 * @param aType
		 * @return Object
		 */
		private Object getValue(String aValue, int aType) {
			if (aValue == null) {
				return null;
			}

			switch (aType) {
				case KILIM.CHAR :
					return new Character(aValue.charAt(0));
				case KILIM.BOOLEAN :
					return new Boolean(aValue);
				case KILIM.BYTE :
					if ("".equals(aValue)) {
						return null;
					}
					return new Byte(Byte.parseByte(aValue));
				case KILIM.SHORT :
					if ("".equals(aValue)) {
						return null;
					}
					return new Short(Short.parseShort(aValue));
				case KILIM.INT :
					if ("".equals(aValue)) {
						return null;
					}
					return new Integer(Integer.parseInt(aValue));
				case KILIM.LONG :
					if ("".equals(aValue)) {
						return null;
					}
					return new Long(Long.parseLong(aValue));
				case KILIM.DOUBLE :
					if ("".equals(aValue)) {
						return null;
					}
					return new Double(Double.parseDouble(aValue));
				case KILIM.FLOAT :
					if ("".equals(aValue)) {
						return null;
					}
					return new Float(Float.parseFloat(aValue));
				case KILIM.STRING :
					return aValue;
				default :
					logParseException("unable to parse Property value " + aValue);
					return null;
			}
		}
		
		private int getStatus(String aStatus, String defaultStatus) {
			if (aStatus == null) {
				aStatus = defaultStatus;
			}
			if ("external".equalsIgnoreCase(aStatus) || "public".equalsIgnoreCase(aStatus)) {
				return KILIM.PUBLIC;
			} else if ("internal".equalsIgnoreCase(aStatus) || "private".equalsIgnoreCase(aStatus)) {
				return KILIM.PRIVATE;
			} else if ("protected".equalsIgnoreCase(aStatus)) {
				return KILIM.PROTECTED;
			} else {
				logParseException("unable to parse status " + aStatus);
			}
			return 0;
		}
		
		private String getCDATA(String cdata) {
			if (cdata == null) {
				return null;
			}
			cdata = cdata.trim();
			if (cdata.startsWith("<![CDATA[")) {
				cdata = cdata.substring(9);
				if (cdata.length() > 3) {
					cdata = cdata.substring(0, cdata.length() - 3);
				}
			}
			return cdata;
		}
				
		private TemplateDescription getCurrentTemplate() {
			return (TemplateDescription) templates.peek();
		}
		
		private TemplateDescription getContainingTemplate() {
			return (TemplateDescription) templates.get(templates.size() - 2);
		}
			
		private Instance getCurrentInstance() {
			return (Instance) instances.peek();
		}
				
		private void logParseException(String message) {
			errorHandler.handleErrorSAXParseException(new SAXParseException(message, locator, null));
		}
		
		// A private method that displays in a warning message the stack state , when a stack problem is discovered.
		//It prints the name of the template beinig parsed followed by the current content of the stack.
		private void displayContextStack(String aMessage, ContextData top) {
			System.err.println("============CONTEXT STACK (begin)================");
			System.err.print("[TEMPLATE : " + currentTemplateName + "]  ");
			System.err.println(aMessage);
			int nbE = contexts.size();
			System.err.println("nb Elem = " + nbE);
			for (int i = 0; i < nbE ; i++) {
				ContextData contextData = (ContextData) contexts.elementAt(i);
				System.err.println("elem " + i + " : phase = " + contextData.parsingPhase + " value = " + contextData.object);
			}
			System.err.println("elem " + nbE + " : phase = " + top.parsingPhase + " value = " + top.object);
			System.err.println("============CONTEXT STACK (end)==================");
		}
	}
}